
#ifndef __CONFIG_KVP_H
#define __CONFIG_KVP_H

#include "hwdec_types.h"
#include "hwdec_return_codes.h"
#include "hwdec_os_api.h"
#include "simple_chain.h"

typedef enum
{
	KVP_INT = 0,
	KVP_STR ,
	KVP_OTHER,
}kvpDataType;

typedef struct 
{
	uint32 key;
	kvpDataType type;
	int val[];
}paramKvp;



class configKvp : public simpleChainSelectionInterface ,
	public simpleChainDataExplainerInterface
{
	public :

		configKvp(){ configChain = new simpleChain(this,this); };

		~configKvp(){ delete configChain; configChain = NULL; };

		RET_STATUS getConfig( paramKvp *kvp , uint32 *len )
		{
			return configChain->GetData( (uint8 *)kvp , len , (void *)&kvp->key );
		};
		
		RET_STATUS setConfig( paramKvp *kvp , uint32 len )
		{
			RET_STATUS ret;

			ret = configChain->ReplaceData( (uint8 *)kvp , len , (void *)&kvp->key);

			if ( ret == Ret_ErrNotFound)
				return configChain->AddToChainTail( (uint8 *)kvp, len);

			return ret;
		};

		RET_STATUS delConfig( uint32 key )
		{
			return configChain->DelFromChain( (void *)&key);
		};

		void delAllConfig( )
		{
			configChain->EmptyChain();
		}

		bool SelectionCallBack( uint8 *data , uint32 dataLen , void * aParam)
		{	
			return (( paramKvp *)data)->key == *((uint32 *)aParam); 
		};

		void ExplainCallBack(uint8 * data, uint32 dataLen)
		{
			paramKvp *pKvp = (paramKvp *)data;

			switch ( pKvp->type )
			{
				case KVP_INT:

					HWDEC_MSG(" 0x%08x = %d size = %d" , pKvp->key , pKvp->val[0] , dataLen);
					break;

				case KVP_STR:

					HWDEC_MSG("0x%08x = %s size = %d" , pKvp->key , (char *)pKvp->val , dataLen);
					break;

				case KVP_OTHER:
				default:
					
					HWDEC_MSG("0x%08x = \"unkown\" size = %d", pKvp->key , dataLen);
					break;
			}
		}

		void DisplayAllConfigDataInfo( )
		{
			configChain->DisplayAllDataInfo();
		}

	private :

		simpleChain *configChain;
};


#endif /* __CONFIG_KVP_H */

