
#ifndef __DEC_PROXY_INTERFACE_H
#define __DEC_PROXY_INTERFACE_H

#include "pv_hwdec_proxied_interface.h"
#include "pv_hwdec_interface_proxy.h"
#include "threadsafe_mempool.h"
#include "pvmf_return_codes.h"
#include "pvmf_event_handling.h"

#include "hwdec_return_codes.h"
#include "hwdec_types.h"
#include "hwdec_factory.h"
#include "hwdec_proc_interface.h"

#define MAX_NUMBER_OF_DEC_PROXY_MSGS		20
#define MAX_SIZE_OF_DEC_PROXY_MSG			128

#define MAX_PENDING_DEC_DATA_CMD			5


#define PROXY_ID_DEFAULT 					1983


#define PROXY_CMD_SETOUTPUTIMG 			0
#define PROXY_CMD_SETINPUTROTATION 		1
#define PROXY_CMD_SETOUTPUTCROP 			2
#define PROXY_CMD_INIT 						3
#define PROXY_CMD_PREPARE   	 				4
#define PROXY_CMD_DECODE					5
#define PROXY_CMD_DEINIT    					6
#define PROXY_CMD_GETHEADERINFO			7	
#define PROXY_CMD_GETHANDLE				8
#define PROXY_CMD_FREEHANDLE				9

#define PROXY_NOTIF_PICTURE_READY			10

typedef uint32 proxyMsgType;

typedef void (*CommandDoneCallback)(DEC_HANDLETYPE , PVMFStatus , TPVProxyMsgId);
typedef void (*NotificationCallback)(DEC_HANDLETYPE , PVMFStatus , TPVProxyMsgId , DEC_U32 , DEC_U32);

class DecProxyInterface : 
	public PVProxiedEngine_HWDEC ,
	public PVProxiedInterfaceServer_HWDEC,
	public PVProxiedInterfaceClient_HWDEC,
	public DecodedDataProcessInterface
{

	public :
		
	DecProxyInterface();
        virtual ~DecProxyInterface();
        DEC_BOOL Start();
        void Exit();
		
        /* PVProxiedEngine Implementation */
	void CreateLoggerAppenders(){};
        void PVThreadLogon(PVMainProxy_HWDEC&);
        void PVThreadLogoff(PVMainProxy_HWDEC&);

	RET_STATUS ProxyGetHandle(
		DEC_HANDLETYPE *pHandle,
		DEC_PTR dpcallback,
		DEC_PTR proxy,
		DEC_PTR appContext,
		CommandDoneCallback callback1,
		NotificationCallback callback2,
		DEC_INPUT_FORMAT in,
		DEC_OUTPUT_FORMAT out);

	RET_STATUS ProxyFreeHandle(
		DEC_HANDLETYPE hComponent);

	/* Proxy Interface for client to use */
	RET_STATUS ProxySetOutputImage(
		DEC_HANDLETYPE hComponent ,
		DEC_U32 nWidth, 
		DEC_U32 nHeight , 
		DEC_U32 nFormat);

	RET_STATUS ProxySetInputRotation(
		DEC_HANDLETYPE hComponent,
		DEC_U32 rotation);

	RET_STATUS ProxySetOutputCrop(
		DEC_HANDLETYPE hComponent,
		DEC_U32 nLeftTopX, 
		DEC_U32 nLeftTopY , 
		DEC_U32 nWidth ,
		DEC_U32 nHeight);

	RET_STATUS ProxyInit(
		DEC_HANDLETYPE hComponent);

	RET_STATUS ProxyPrepare(
		DEC_HANDLETYPE hComponent);

	RET_STATUS ProxyDecode(
		DEC_HANDLETYPE hComponent ,
		DEC_UCHAR *pInputData , 
		DEC_U32 nInputDataLen , 
		DEC_BOOL bEos);

	RET_STATUS ProxyDeinit(
		DEC_HANDLETYPE hComponent);

	RET_STATUS ProxyGetHeaderInfo(
		DEC_HANDLETYPE hComponent,
		DEC_U32 *pFrameWidth, 
		DEC_U32 *pFrameHeight, 
		DEC_U32 *pDisplayWidth, 
		DEC_U32 *pDisplayHeight, 
		DEC_U32 *pDecOutFormat);

	TPVProxyMsgId GetCurrentAsyncCommandId() { return iCurrentMsgId; };

	/*  
		PVProxiedInterfaceServer Implementation 
	*/
	void HandleCommand(TPVProxyMsgId aId, OsclAny* aMsg);
	void CleanupNotification(TPVProxyMsgId aId, OsclAny* aMsg) ;

	/*	
		PVProxiedInterfaceClient Implementation 
	*/
	void HandleNotification(TPVProxyMsgId aId, OsclAny* aMsg);
	void CleanupCommand(TPVProxyMsgId aId, OsclAny* aMsg) ;

	/*
		DecodedDataProcessInterface
	*/
	RET_STATUS OnePictureReadyCallback(DEC_UCHAR * pData, DEC_U32 nLen, DEC_PTR pParam);

	OsclAny *GetMemPoolPtr()
        {
        	return (OsclAny*) iMemoryPool;
        };
	/*
		Server method pointer
	*/
	RET_STATUS (*ComponentSetOutputImage)(
		DEC_HANDLETYPE hComponent, 
		DEC_U32 nWidth , 
		DEC_U32 nHeight , 
		DEC_U32 nOutputFormat) ;

	RET_STATUS (*ComponentSetInputRotation)( 
		DEC_HANDLETYPE hComponent , 
		DEC_U32 nRotation);

	RET_STATUS (*ComponentSetOutputCrop)( 
		DEC_HANDLETYPE hComponent,
		DEC_U32 nLeftTopX, 
		DEC_U32 nLeftTopY , 
		DEC_U32 nWidth , 
		DEC_U32 nHeight);

	RET_STATUS (*ComponentInit)(
		DEC_HANDLETYPE hComponent);

	RET_STATUS (*ComponentPrepare)(
		DEC_HANDLETYPE hComponent);

	RET_STATUS (*ComponentDeinit)(
		DEC_HANDLETYPE hComponent);

	RET_STATUS (*ComponentDecode)(
		DEC_HANDLETYPE hComponent,
		DEC_UCHAR * pInputData, 
		DEC_U32 nInputDataLen, 
		DEC_BOOL bEos);

	RET_STATUS (*ComponentGetHeaderInfo)(
		DEC_HANDLETYPE hComponent,
		DEC_U32 *aFrameWidth, 
		DEC_U32 *aFrameHeight, 
		DEC_U32 *aDisplayWidth, 
		DEC_U32 *aDisplayHeight , 
		DEC_U32 *aDecOutFormat);	


	/*
		Client method pointer
	*/
	void (*HandleCommandDoneCallback)(
		DEC_HANDLETYPE hComponent , 
		PVMFStatus status , 
		TPVProxyMsgId cmdId);

	void (*HandleOneFrameDecodedCallback)(
		DEC_HANDLETYPE hComponent , 
		PVMFStatus status , 
		TPVProxyMsgId cmdId ,
		DEC_U32 param1,
		DEC_U32 param2);

private:

	RET_STATUS iSynCmdRet;
	TPVProxyId iProxyId;
	OsclSemaphore iCmdSem;
	CPVInterfaceProxy_HWDEC *iProxy;
	TPVProxyMsgId iCurrentMsgId;
	TPVProxyMsgId iCurrentHandlingMsgId;
	bool iReadyToDecode;
	DEC_U32 iMemCmd;
	DEC_U32 iDecDataCmd;
	DEC_U32 iDecControlCmd;
	DEC_U32 iNumCreate;
	DEC_U32 iNumDestroy;
	ThreadSafeMemPoolFixedChunkAllocator *iMemoryPool;
};


class MsgBase
{
	public:
		MsgBase(proxyMsgType msg)
			:iMsg(msg)
		{}
		~MsgBase()
		{}

		proxyMsgType getMsg(){ return iMsg; }

		proxyMsgType iMsg;
};

class GetDecHandleMsg : public MsgBase
{
	public:
		GetDecHandleMsg(DEC_HANDLETYPE *p , DEC_PTR dp,
			DEC_PTR proxy, DEC_PTR ctx, 
			CommandDoneCallback callback1, NotificationCallback callback2 , 
			DEC_INPUT_FORMAT in , DEC_OUTPUT_FORMAT out )
			
			:MsgBase(PROXY_CMD_GETHANDLE), 
			pHandle(p), pDp(dp), pProxy(proxy), pAppContext(ctx), 
			iCallback1(callback1), iCallback2(callback2),
			inFmt(in) , outFmt(out)
		{};
		~GetDecHandleMsg()
		{};

		DEC_HANDLETYPE *pHandle;
		DEC_PTR pDp;
		DEC_PTR pProxy;
		DEC_PTR pAppContext;
		CommandDoneCallback iCallback1;
		NotificationCallback iCallback2;
		DEC_INPUT_FORMAT inFmt;
		DEC_OUTPUT_FORMAT outFmt;
};


class FreeDecHandleMsg : public MsgBase
{
	public: 
		FreeDecHandleMsg(DEC_HANDLETYPE h)
			:MsgBase(PROXY_CMD_FREEHANDLE), 
			hComponent(h)
		{}
		~FreeDecHandleMsg()
		{}

		DEC_HANDLETYPE hComponent;
};


class SetOutputImgMsg : public MsgBase
{
	public:
		SetOutputImgMsg(DEC_HANDLETYPE h , DEC_U32 width , DEC_U32 height, DEC_U32 format)
			:MsgBase(PROXY_CMD_SETOUTPUTIMG),
			hComponent(h), nWidth(width) , nHeight(height) , nFormat(format)
		{}
	 	~SetOutputImgMsg()
		{}

		DEC_HANDLETYPE hComponent;
		DEC_U32 nWidth;
		DEC_U32 nHeight;
		DEC_U32 nFormat;
};


class SetInputRotationMsg : public MsgBase
{
	public:
		SetInputRotationMsg(DEC_HANDLETYPE h , DEC_U32 rotation)
			:MsgBase(PROXY_CMD_SETINPUTROTATION),
			hComponent(h) , nRotation(rotation)
		{}
		~SetInputRotationMsg()
		{}

		DEC_HANDLETYPE hComponent;
		DEC_U32 nRotation;
};


class SetOutputCropMsg : public MsgBase
{
	public:
		SetOutputCropMsg(DEC_HANDLETYPE h , DEC_U32 x , DEC_U32 y , DEC_U32 width , DEC_U32 height)
			: MsgBase(PROXY_CMD_SETOUTPUTCROP),
			hComponent(h) , nTopLeftX(x) , nTopLeftY(y) , nWidth(width) , nHeight(height)
		{}
		~SetOutputCropMsg()
		{}

		DEC_HANDLETYPE hComponent;
		DEC_U32 nTopLeftX;
		DEC_U32 nTopLeftY;
		DEC_U32 nWidth;
		DEC_U32 nHeight;
};


class InitMsg : public MsgBase
{
	public: 
		InitMsg(DEC_HANDLETYPE h)
			: MsgBase(PROXY_CMD_INIT), 
			hComponent(h)
		{}
		~InitMsg()
		{}

		DEC_HANDLETYPE hComponent;
};


class PrepareMsg : public MsgBase
{
	public:
		PrepareMsg(DEC_HANDLETYPE h)
			: MsgBase(PROXY_CMD_PREPARE), 
			hComponent(h)
			
		{}
		~PrepareMsg()
		{}

		DEC_HANDLETYPE hComponent;
};


class DecodeMsg : public MsgBase
{
	public:
		DecodeMsg(DEC_HANDLETYPE h, DEC_UCHAR * pInputData, DEC_U32 nInputDataLen, DEC_BOOL eos)
			: MsgBase(PROXY_CMD_DECODE), 
			hComponent(h), pData(pInputData), nDataLen(nInputDataLen), bEos(eos)
		{}
		~DecodeMsg()
		{}

		DEC_HANDLETYPE hComponent;
		DEC_UCHAR *pData;
		DEC_U32	nDataLen;
		DEC_BOOL bEos;
};


class DeinitMsg : public MsgBase
{
	public:
		DeinitMsg(DEC_HANDLETYPE h)
			:MsgBase(PROXY_CMD_DEINIT),
			hComponent(h)
		{}
		~DeinitMsg()
		{}

		DEC_HANDLETYPE hComponent;
};


class GetHeaderInfoMsg : public MsgBase
{
	public:
		GetHeaderInfoMsg(DEC_HANDLETYPE h, DEC_U32 *pFrameWidth, DEC_U32 *pFrameHeight, 
						DEC_U32 *pDisplayWidth, DEC_U32 *pDisplayHeight, DEC_U32 *pDecOutFormat)
			: MsgBase(PROXY_CMD_GETHEADERINFO),
			hComponent(h), pFrameW(pFrameWidth), pFrameH(pFrameHeight), pDisplayW(pDisplayWidth),
			pDisplayH(pDisplayHeight), pFormat(pDecOutFormat)
		{}
		~GetHeaderInfoMsg()
		{}

		DEC_HANDLETYPE hComponent;
		DEC_U32 *pFrameW;
		DEC_U32 *pFrameH;
		DEC_U32 *pDisplayW;
		DEC_U32 *pDisplayH;
		DEC_U32 *pFormat;
};


class CompleteNotification
{
	public:
		CompleteNotification(TPVProxyMsgId cmdid ,proxyMsgType msg, RET_STATUS status,
								DEC_U32 param1,DEC_U32 param2,DEC_U32 param3)
			: nCmdId(cmdid), nMsg(msg), nStatus(status), 
			nParam1(param1), nParam2(param2), nParam3(param3)
		{}
		~CompleteNotification()
		{}

		TPVProxyMsgId nCmdId;
		proxyMsgType nMsg;
		RET_STATUS nStatus;
		DEC_U32 nParam1;
		DEC_U32 nParam2;
		DEC_U32 nParam3;
};

#endif	/*	__DEC_PROXY_INTERFACE_H  */

