
#ifndef __HWDEC_BASE_IMP_H
#define __HWDEC_BASE_IMP_H

#include "pvmf_timestamp.h"
#include "pvmf_return_codes.h"
#include "pv_hwdec_proxied_interface.h"

#include "hwdec_interface.h"
#include "hwdec_post_proc.h"
#include "hwdec_proc_interface.h"
#include "linear_malloc.h"


#define VIDEO_DEFAULT_BUFFER_NUM  	4
#define VIDEO_DEFAULT_BUFFER_GROW 	2
#define VIDEO_DEFAULT_BUFFER_SIZE	(2*1024*1024) /* 2 megabytes for init buffer size */ 

#define VIDEO_INPUT_BUFFER_SIZE		(1*1024*1024)
#define VIDEO_OUTPUT_BUFFER_SIZE	(2*1024*1024)
#define VIDEO_OUTPUT_BUFFER_NUM	3

#define LBF_HANDLER_INVALID			(-1)

#define __DEC_ENTRY_POINT static

typedef enum
{
	DEC_RET_OK = 0,
	DEC_RET_HDR_READY ,
	DEC_RET_PIC_DECODED,
	DEC_RET_STREAM_PROCESSED,
	DEC_RET_STREAM_ERR,
	DEC_RET_STREAM_NOT_SUPPORTED
}DEC_RET;


class DecComponentBase
{
	public :

		DecComponentBase();

		virtual ~DecComponentBase(){};

		void ConstructComponent(
			DEC_PTR pProxy, DEC_PTR appContext, DEC_PTR decoderContext, 
			void (*callback1)(DEC_HANDLETYPE , PVMFStatus , TPVProxyMsgId ),
			void (*callback2)(DEC_HANDLETYPE , PVMFStatus , TPVProxyMsgId , DEC_U32 , DEC_U32));

		void SetDecodedDataProcInterface( DecodedDataProcessInterface *dp , DEC_PTR param)
		{
			HWDEC_MSG("dp = 0x%08x . param = 0x%08x" , dp , param);
			iDpInf = dp;
			iDpParam = param;
		}

		/* 
			static function for the entry point without proxy 
		*/
		__DEC_ENTRY_POINT RET_STATUS BaseComponentSetOutputImage(
			DEC_HANDLETYPE hComponent, 
			DEC_U32 nWidth , 
			DEC_U32 nHeight , 
			DEC_U32 nOutputFormat) ;

		__DEC_ENTRY_POINT RET_STATUS BaseComponentSetInputRotation( 
			DEC_HANDLETYPE hComponent , 
			DEC_U32 nRotation);

		__DEC_ENTRY_POINT RET_STATUS BaseComponentSetOutputCrop( 
			DEC_HANDLETYPE hComponent,
			DEC_U32 nLeftTopX, 
			DEC_U32 nLeftTopY , 
			DEC_U32 nWidth , 
			DEC_U32 nHeight);

		__DEC_ENTRY_POINT RET_STATUS BaseComponentInit(
			DEC_HANDLETYPE hComponent);

		__DEC_ENTRY_POINT RET_STATUS BaseComponentPrepare(
			DEC_HANDLETYPE hComponent);

		__DEC_ENTRY_POINT RET_STATUS BaseComponentDeinit(
			DEC_HANDLETYPE hComponent);

		__DEC_ENTRY_POINT RET_STATUS BaseComponentDecode(
			DEC_HANDLETYPE hComponent,
			DEC_UCHAR * pInputData, 
			DEC_U32 nInputDataLen, 
			DEC_BOOL bEos);

		__DEC_ENTRY_POINT RET_STATUS BaseComponentGetHeaderInfo(
			DEC_HANDLETYPE hComponent,
			DEC_U32 *pFrameWidth, 
			DEC_U32 *pFrameHeight, 
			DEC_U32 *pDisplayWidth, 
			DEC_U32 *pDisplayHeight , 
			DEC_U32 *pDecOutFormat);		

		__DEC_ENTRY_POINT TPVProxyId BaseComponentGetCommandId(
			DEC_HANDLETYPE hComponent);


		/* 
			static function for the entry point with proxy 
		*/
		__DEC_ENTRY_POINT RET_STATUS BaseComponentProxySetOutputImage(
			DEC_HANDLETYPE hComponent, 
			DEC_U32 nWidth , 
			DEC_U32 nHeight , 
			DEC_U32 nOutputFormat) ;

		__DEC_ENTRY_POINT RET_STATUS BaseComponentProxySetInputRotation( 
			DEC_HANDLETYPE hComponent , 
			DEC_U32 nRotation);

		__DEC_ENTRY_POINT RET_STATUS BaseComponentProxySetOutputCrop( 
			DEC_HANDLETYPE hComponent,
			DEC_U32 nLeftTopX, 
			DEC_U32 nLeftTopY , 
			DEC_U32 nWidth , 
			DEC_U32 nHeight);

		__DEC_ENTRY_POINT RET_STATUS BaseComponentProxyInit(
			DEC_HANDLETYPE hComponent);

		__DEC_ENTRY_POINT RET_STATUS BaseComponentProxyPrepare(
			DEC_HANDLETYPE hComponent);

		__DEC_ENTRY_POINT RET_STATUS BaseComponentProxyDeinit(
			DEC_HANDLETYPE hComponent);

		__DEC_ENTRY_POINT RET_STATUS BaseComponentProxyDecode(
			DEC_HANDLETYPE hComponent,
			DEC_UCHAR * pInputData, 
			DEC_U32 nInputDataLen, 
			DEC_BOOL bEos);

		__DEC_ENTRY_POINT RET_STATUS BaseComponentProxyGetHeaderInfo(
			DEC_HANDLETYPE hComponent,
			DEC_U32 *pFrameWidth, 
			DEC_U32 *pFrameHeight, 
			DEC_U32 *pDisplayWidth, 
			DEC_U32 *pDisplayHeight , 
			DEC_U32 *pDecOutFormat);		

		__DEC_ENTRY_POINT TPVProxyId BaseComponentProxyGetCommandId(
			DEC_HANDLETYPE hComponent);

		__DEC_ENTRY_POINT RET_STATUS BaseComponentReturnBuffer(
			DEC_HANDLETYPE hComponent);

		TPVProxyMsgId GetAsyncCommandId(){ return iCurrentAsyncCommandId; };

		/* Native function implementation */
		RET_STATUS SetOutputImage( 
			DEC_U32 nWidth , 
			DEC_U32 nHeight , 
			DEC_U32 nOutputFormat ) ;

		RET_STATUS SetInputRotation( uint32 nRotation );

		RET_STATUS SetOutputCrop( 
			DEC_U32 nTopLeftX, 
			DEC_U32 nTopLeftY , 
			DEC_U32 nWidth , 
			DEC_U32 nHeight);

		RET_STATUS Init();

		RET_STATUS Prepare();

		void Deinit();

		RET_STATUS Decode( 
			DEC_UCHAR *pInputData, 
			DEC_U32 nInputDataLen, 
			DEC_BOOL bEos);

		RET_STATUS GetHeaderInfo(
			DEC_U32 *pFrameWidth, 
			DEC_U32 *pFrameHeight, 
			DEC_U32 *pDisplayWidth, 
			DEC_U32 *pDisplayHeight , 
			DEC_U32 *pDecOutFormat);	


		/* 
			Decoder specific interface :
			MPEG4 , H264 , VC1 .etc should be derived from this interface 
		*/
		virtual RET_STATUS DecSpecificInit() = 0;

		virtual RET_STATUS DecSpecificPrepare() = 0;

		virtual RET_STATUS DecSpecificDeinit() = 0;

		virtual RET_STATUS GetVideoInfo( 
			DEC_U32 &aImgFormat , 
			DEC_U32 &aDisplayWidth , 
			DEC_U32 &aDisplayHeight ,
			DEC_U32 &aFrameWidth , 
			DEC_U32 &aFrameHeight , 
			DEC_U32 &aInterlaced) = 0;
		
		virtual DEC_RET VideoDecode( 
			DEC_UCHAR *pData , 
			DEC_U32 nBusAddr , 
			DEC_U32 nDataLen , 
			DEC_UCHAR **ppOutData , 
			DEC_U32 *pOutLen , 
			DEC_U32 *pLeft, 
			DEC_BOOL bEos) = 0;


		virtual DEC_HANDLER DecSpecificGetHandler() = 0;

		virtual DEC_U32 DecSpecificGetPPMode() = 0;

		DEC_PTR GetProxyInterface(){ return iProxyApp; };

		DEC_HANDLETYPE GetComponentHandle(){ return (DEC_HANDLETYPE)&iComponent; };

		/*
			Output buffer control method
		*/
		RET_STATUS OutputBufferOpen(uint32 basePA, uint8 * baseVA, uint32 bufferSize, uint bufferNum);
		void OutputBufferClose();
		RET_STATUS OutputBufferGetNext(uint32 & pa, uint8 * & va, uint32 & size);
		uint32 OutputBufferIndexInc();
		uint32 OutputBufferIndexDec();
		RET_STATUS OutputBufferRetrun();

		/* 
			Protected memebers could be accessed by derived class
			Most of them are decoding process or post processor related data
			Could be used by derived class
		*/
	protected:

		RET_STATUS DecodeInternal( DEC_UCHAR * pVa, DEC_U32 nPa, DEC_U32 nSize, DEC_BOOL bEos);

		enum {
			DEC_STAT_IDLE = 0,
			DEC_STAT_INITIALIZED,
			DEC_STAT_PREPARED,
			DEC_STAT_HEADER_READY,
			DEC_STAT_DECODING,
			DEC_STAT_PAUSED,
			DEC_STAT_STOP,
			DEC_STAT_ERR
		}iState;

		DecodedDataProcessInterface *iDpInf;
		DEC_PTR iDpParam;

		HWPostProcessor *pp;
		LinearBufferAllocator *LBAllocator;
		LBUF_HANDLER inputBuffer , outputBuffer;

		uint32 iOutputBufferBasePA;
		uint8 *iOutputBufferBaseVA;
		uint32 *iOutputBufferOffset;
		uint32 iOutputBufferNum;
		uint32 iOutputBufferIndex;
		OsclSemaphore *iOutputBufferSem;
		bool iOutputBufferUpdate;
		uint32 iCurrentBufferPA;
		uint8 *iCurrentBufferVA;
		uint32 iCurrentBufferSize;
		
		DEC_BOOL iPeerConnected;

		/* Post processor parameters */
		DEC_U32 iCropWidth;
		DEC_U32 iCropHeight;
		DEC_U32 iCropX;
		DEC_U32 iCropY;
		DEC_U32 iOutputWidth;
		DEC_U32 iOutputHeight;
		DEC_U32 iRotation;

		DEC_U32 iInDataFormat;
		/* Actual width of video data */
		DEC_U32 iDisplayWidth;
		/* Actual height of video data */
		DEC_U32 iDisplayHeight;
		/* Padding frame width , muptiple of 16 pixels */
		DEC_U32 iFrameWidth;
		/* Padding frame height , multiple of 16 pixels */
		DEC_U32 iFrameHeight;

		DEC_U32 iInterlaced;

		DEC_U32 iDecOutFormat;
		/* Output data format */
		DEC_U32 iImgOutFormat;

		DEC_COMPONENTTYPE iComponent;
		DEC_PTR iProxyApp;
		TPVProxyMsgId iCurrentAsyncCommandId;
};



#endif /* __HWDEC_BASE_IMP_H */

