
#ifndef __HW_DEC_INTERFACE_H
#define __HW_DEC_INTERFACE_H

#include "hwdec_return_codes.h"
#include "hwdec_types.h"
#include "dec_proxy_interface.h"

typedef enum
{
	V_OUT_YUV = 0 ,
	V_OUT_RGB15 , 
	V_OUT_RGB16,
	V_OUT_RGB32,
}VOUTFMT;

typedef struct DecoderComponent_s
{
	/* 	Always point back to decComponentBase */
	DEC_PTR componentPrivateData;

	/* 	Always point to the app */
	DEC_PTR applicationPrivateData;

	/* 	Decoder specific data */
	DEC_PTR decoderPrivateData;

	/*	Below are the component methods provided to application */
	RET_STATUS (*DEC_SetOutputImage)(
		DEC_HANDLETYPE hComponent,
		DEC_U32 nWidth , 
		DEC_U32 nHeight , 
		DEC_U32 nOutputFormat);

	RET_STATUS (*DEC_SetRatation)(
		DEC_HANDLETYPE hComponent , 
		DEC_U32 nRotation);

	RET_STATUS (*DEC_SetOutputCrop)(
		DEC_HANDLETYPE hComponent,
		DEC_U32 nLeftTopX, 
		DEC_U32 nLeftTopY , 
		DEC_U32 nWidth , 
		DEC_U32 nHeight);

	RET_STATUS (*DEC_Init)(
		DEC_HANDLETYPE hComponent);

	RET_STATUS (*DEC_Prepare)(
		DEC_HANDLETYPE hComponent);

	RET_STATUS (*DEC_Decode)(
		DEC_HANDLETYPE hComponent ,
		DEC_UCHAR *pInputData , 
		DEC_U32 nInputDataLen , 
		DEC_BOOL bEos);

	RET_STATUS (*DEC_Deinit)(
		DEC_HANDLETYPE hComponent);

	RET_STATUS (*DEC_GetHeaderInfo)(
		DEC_HANDLETYPE hComponent,
		DEC_U32 *pFrameWidth, 
		DEC_U32 *pFrameHeight, 
		DEC_U32 *pDisplayWidth, 
		DEC_U32 *pDisplayHeight , 
		DEC_U32 *pDecOutFormat);

	TPVProxyId (*DEC_GetCommandId)(
		DEC_HANDLETYPE hComponent);

	RET_STATUS (*DEC_ReturnBuffer)(
		DEC_HANDLETYPE hComponent);
	
}DEC_COMPONENTTYPE;


#define HWDEC_SetOutputImg(	\
	hComponent,	\
	width,	\
	height,	\
	format)	\
((DEC_COMPONENTTYPE*)hComponent)->DEC_SetOutputImage(       \
	hComponent,	\
	width,	\
	height,	\
	format)


#define HWDEC_SetInputRotation(	\
	hComponent,	\
	nRotation)	\
((DEC_COMPONENTTYPE*)hComponent)->DEC_SetInputRotation(       \
	hComponent,	\
	nRotation)


#define HWDEC_SetOutputCrop(	\
	hComponent,	\
	nLeftTopX,	\
	nLeftTopY,	\
	nWidth,	\
	nHeight,	\
	nFormat)	\
((DEC_COMPONENTTYPE*)hComponent)->DEC_SetOutputCrop(       \
	hComponent,	\
	nLeftTopX,	\
	nLeftTopY,	\
	nWidth,	\
	nHeight,	\
	nFormat)


#define HWDEC_Init(	\
	hComponent)	\
((DEC_COMPONENTTYPE*)hComponent)->DEC_Init(       \
	hComponent)


#define HWDEC_Prepare(	\
	hComponent)	\
((DEC_COMPONENTTYPE*)hComponent)->DEC_Prepare(       \
	hComponent)


#define HWDEC_Decode(	\
        hComponent,	 \
        pBuffer,	\
        nLen,	\
        bEos)	\
((DEC_COMPONENTTYPE*)hComponent)->DEC_Decode(       \
        hComponent,	\
        pBuffer,	\
	nLen,	\
	bEos)				                        /* Macro End */


#define HWDEC_Deinit(	\
	hComponent)	\
((DEC_COMPONENTTYPE*)hComponent)->DEC_Deinit(       \
	hComponent)


#define HWDEC_GetHeaderInfo(	\
	hComponent,	\
	pFrameWidth,	\
	pFrameHeight,	\
	pDisplayWidth,	\
	pDisplayHeight,	\
	pDecOutFormat)	\
((DEC_COMPONENTTYPE*)hComponent)->DEC_GetHeaderInfo(       \
	hComponent,	\
	pFrameWidth,	\
	pFrameHeight,	\
	pDisplayWidth,	\
	pDisplayHeight,	\
	pDecOutFormat)

#define HWDEC_GetCommandId(	\
	hComponent)	\
((DEC_COMPONENTTYPE*)hComponent)->DEC_GetCommandId(       \
	hComponent)

#define HWDEC_ReturnBuffer(	\
	hComponent)	\
((DEC_COMPONENTTYPE*)hComponent)->DEC_ReturnBuffer(       \
	hComponent)

#endif /* __ATMEL_DEC_INTERFACE_H */


