
#ifndef __HWDEC_POST_PROC_H
#define __HWDEC_POST_PROC_H

#include "hwdec_return_codes.h"
#include "hwdec_types.h"
#include "ppapi.h"
#include "config_kvp.h"

typedef PPInst PP_HANDLER;
typedef void * DEC_HANDLER;

typedef enum
{
	/* For input image param */
	PP_INIMG_FORMAT = 0 ,
	PP_INIMG_PIC_STRUCT,
	PP_IMIMG_VIDEO_RAGE,
	PP_INIMG_WIDTH,
	PP_INIMG_HEIGHT,
	PP_INIMG_PA,
	PP_INIMG_CB_PA,
	PP_INIMG_CR_PA,
	PP_INIMG_VC1_MULTI_RES_EN,
	PP_INIMG_VC1_RANGE_RED_FRM,
	PP_INIMG_VC1_RANGE_MAPY_EN,
	PP_INIMG_VC1_RANGE_MAPY_COEFF,
	PP_INIMG_VC1_RANGE_MAPC_EN,
	PP_INIMG_VC1_RANGE_MAPC_COEFF,
	
	/* For crop param */
	PP_CROP_WIDTH,
	PP_CROP_HEIGHT,
	PP_CROP_X,
	PP_CROP_Y,
	PP_CROP_EN,

	/* For rotation param */
	PP_ROTA_ROTATION ,

	/* For output imgage param */
	PP_OUTIMG_FORMAT,
	PP_OUTIMG_WIDTH,
	PP_OUTIMG_HEIGHT,
	PP_OUTIMG_PA,
	PP_OUTIMG_CHROM_PA,
	PP_OUTIMG_DEINTERLACE,
	
}PPConfigKey;

class HWPostProcessor
{
	public :

		HWPostProcessor();
		~HWPostProcessor();

		/* Call this function to init the hardware component */
		RET_STATUS Init();

		/* Disable the hardware engine  */
		RET_STATUS Deinit();

		/* Combine a post processor with a specific decoder */
		RET_STATUS Connect( DEC_HANDLER decHandler , uint32 decType );

		/* Disable the combination  */
		RET_STATUS Disconnect( DEC_HANDLER decHandler );

		/* Set Config */
		RET_STATUS SetConfig( uint32 id , uint32 val);

		/* Get Config */
		RET_STATUS GetConfig( uint32 id , uint32 &val);

		/* Get Picture size */
		uint32 getCurrentPicSize();

		void DumpConfig();

		bool SetOutputBufferAddrAndUpdate(uint32 pa);

		/* SetConfig would take effect immediately , you should call UpdateConfig to make 
		Change taking effect */
		RET_STATUS Updateconfig( uint32 pa , uint32 bufferSize, uint32 &reqSize );

	private :

		void InputRotationParamCheck(uint32 & rotation, uint32 & mWidth, uint32 & mHeight);

		void OutputImgParamCheck(uint32 width, uint32 height, uint32 & outWidth, uint32 & outHeight);

		uint32 iOutImgSize;
		configKvp configData;
		PP_HANDLER ppHandler;
		DEC_HANDLER decHandler;

		bool iConnected;
};


#endif /* __HWDEC_POST_PROC_H */

