
#ifndef __SIMPLE_CHAIN_H
#define __SIMPLE_CHAIN_H

#include "hwdec_return_codes.h"
#include "hwdec_types.h"


typedef struct s_chainEntry 
{
	struct s_chainEntry *next;
	uint32 dataLen;
	void *data;
}chainEntry;

/* It's up to the up layer to make the decision which data should be picked up 
for processing */
class simpleChainSelectionInterface
{
	public :
		
		virtual bool SelectionCallBack( uint8 *data , uint32 dataLen , void *aParam) = 0;
		virtual ~simpleChainSelectionInterface(){}
};

/* From the standpoint of simple chain , the data stored is nothing more than a black box
So, it's necessary to provide a explainer by derive class to know how to explain the data
It's very useful when want to see the data content */
class simpleChainDataExplainerInterface
{
	public :

		virtual void ExplainCallBack(uint8 *data , uint32 dataLen ) = 0;
		virtual ~simpleChainDataExplainerInterface(){}
};

class simpleChain
{
	public:

		simpleChain();

		simpleChain(simpleChainSelectionInterface *selector ,
						simpleChainDataExplainerInterface *explainer);

		~simpleChain();

		RET_STATUS AddToChainTail( uint8 *data , uint32 dataLen );
		
		RET_STATUS AddToChainHead(uint8 * data, uint32 dataLen);

		RET_STATUS DelFromChain(void *aParam);

		RET_STATUS GetData(uint8 * pData, uint32 *dataLen, void * aParam);

		RET_STATUS ReplaceData(uint8 * data, uint32 dataLen, void * aParam);

		void EmptyChain();

		void SetChainSelector( simpleChainSelectionInterface *selector);

		void SetDataExplainer( simpleChainDataExplainerInterface *explainer);

		void DisplayAllDataInfo();

	private :

		simpleChainSelectionInterface *iSelector; 
		simpleChainDataExplainerInterface *iExplainer;
		chainEntry * AllocateChainEntry( uint8 *data, uint32 dataLen );
		
		chainEntry *chainHead;
		uint32 iChainEntryNum;
};


#endif /* __SIMPLE_CHAIN_H */

