


#include "dec_proxy_interface.h"
#include "hwdec_base_imp.h"

extern DEC_HANDLETYPE CreateDecoder(DecodedDataProcessInterface * dp, DecProxyInterface * proxy, 
	DEC_PTR pAppData, CommandDoneCallback callback1, NotificationCallback callback2, DEC_INPUT_FORMAT inFmt, DEC_OUTPUT_FORMAT outFmt);

extern void DestructDecoder(DEC_HANDLETYPE hComponent);


DecProxyInterface::DecProxyInterface()
{

	iMemCmd = 0;
	iDecDataCmd = 0;
	iDecControlCmd = 0;
	iNumCreate = 0;
	iNumDestroy = 0;
	iProxy = NULL;
	iProxyId = PROXY_ID_DEFAULT;
	iReadyToDecode = false;
	iCmdSem.Create();
	iMemoryPool = NULL;
	iMemoryPool = ThreadSafeMemPoolFixedChunkAllocator::Create(MAX_NUMBER_OF_DEC_PROXY_MSGS);

	if (iMemoryPool != NULL)
	{
		OsclAny *dummy = iMemoryPool->allocate(MAX_SIZE_OF_DEC_PROXY_MSG * sizeof(uint8));
		iMemoryPool->deallocate(dummy);
    	}
}


DecProxyInterface::~DecProxyInterface()
{
	iCmdSem.Close();

	if (iMemoryPool)
    	{
		iMemoryPool->removeRef();
		iMemoryPool = NULL;
	}
}


DEC_BOOL DecProxyInterface::Start()
{
	iProxy = CPVInterfaceProxy_HWDEC::NewL(*this);

	if (!iProxy)
	{
		return false;
	}

	return iProxy->StartPVThread();
}


void DecProxyInterface::Exit()
{
	if (NULL == iProxy)
	{
        	return;
	}

	/*
		this will stop proxy scheduler and thread.
	*/
	iProxy->Delete();
	iProxy = NULL;
}


void DecProxyInterface::PVThreadLogon(PVMainProxy_HWDEC &proxy)
{
	iNumCreate++;
    	iProxyId = proxy.RegisterProxiedInterface(*this, *this);
}


void DecProxyInterface::PVThreadLogoff(PVMainProxy_HWDEC &proxy)
{
	iNumDestroy++;
	proxy.UnregisterProxiedInterface(iProxyId);
}


RET_STATUS DecProxyInterface::ProxyGetHandle(
	DEC_HANDLETYPE * pHandle, DEC_PTR dp,
	DEC_PTR proxy, DEC_PTR appContext, 
	CommandDoneCallback callback1,  NotificationCallback callback2,
	DEC_INPUT_FORMAT in , DEC_OUTPUT_FORMAT out)
{
	OsclAny *ptr = iMemoryPool->allocate(MAX_SIZE_OF_DEC_PROXY_MSG * sizeof(uint8));

	if (NULL == ptr)
	{
        	return Ret_ErrNoResources;
	}

    	GetDecHandleMsg* Msg = OSCL_PLACEMENT_NEW(ptr, 
			GetDecHandleMsg(pHandle, dp, proxy , appContext , callback1, callback2, in, out));

	iMemCmd++;
    	iDecControlCmd++;
    	iCurrentMsgId = iProxy->SendCommand(iProxyId, (OsclAny*) Msg);
    	iCmdSem.Wait();

    	return iSynCmdRet;
}


RET_STATUS DecProxyInterface::ProxyFreeHandle(DEC_HANDLETYPE hComponent)
{
	OsclAny *ptr = iMemoryPool->allocate(MAX_SIZE_OF_DEC_PROXY_MSG * sizeof(uint8));

	if (NULL == ptr)
	{
        	return Ret_ErrNoResources;
	}

    	FreeDecHandleMsg* Msg = OSCL_PLACEMENT_NEW(ptr, 
			FreeDecHandleMsg( hComponent));

	iMemCmd++;
    	iDecControlCmd++;
    	iCurrentMsgId = iProxy->SendCommand(iProxyId, (OsclAny*) Msg);
    	iCmdSem.Wait();

    	return iSynCmdRet;
}

RET_STATUS DecProxyInterface::ProxySetOutputImage(
	DEC_HANDLETYPE hComponent, DEC_U32 nWidth, DEC_U32 nHeight, DEC_U32 nFormat)
{
	OsclAny *ptr = iMemoryPool->allocate(MAX_SIZE_OF_DEC_PROXY_MSG * sizeof(uint8));

	if (NULL == ptr)
	{
        	return Ret_ErrNoResources;
	}

    	SetOutputImgMsg* Msg = OSCL_PLACEMENT_NEW(ptr, 
			SetOutputImgMsg(hComponent, nWidth , nHeight , nFormat));

	iMemCmd++;
    	iDecControlCmd++;
    	iCurrentMsgId = iProxy->SendCommand(iProxyId, (OsclAny*) Msg);
    	iCmdSem.Wait();

    	return iSynCmdRet;
}


RET_STATUS DecProxyInterface::ProxySetInputRotation(
	DEC_HANDLETYPE hComponent, DEC_U32 rotation)
{
	OsclAny *ptr = iMemoryPool->allocate(MAX_SIZE_OF_DEC_PROXY_MSG * sizeof(uint8));

	if (NULL == ptr)
	{
        	return Ret_ErrNoResources;
	}

    	SetInputRotationMsg* Msg = OSCL_PLACEMENT_NEW(ptr, 
			SetInputRotationMsg(hComponent, rotation));

	iMemCmd++;
    	iDecControlCmd++;
    	iCurrentMsgId = iProxy->SendCommand(iProxyId, (OsclAny*) Msg);
    	iCmdSem.Wait();

    	return iSynCmdRet;
}


RET_STATUS DecProxyInterface::ProxySetOutputCrop(
	DEC_HANDLETYPE hComponent, DEC_U32 nLeftTopX, DEC_U32 nLeftTopY, DEC_U32 nWidth, DEC_U32 nHeight)
{
	OsclAny *ptr = iMemoryPool->allocate(MAX_SIZE_OF_DEC_PROXY_MSG * sizeof(uint8));

	if (NULL == ptr)
	{
        	return Ret_ErrNoResources;
	}

    	SetOutputCropMsg* Msg = OSCL_PLACEMENT_NEW(ptr, 
		SetOutputCropMsg(hComponent, nLeftTopX , nLeftTopY , nWidth , nHeight));

	iMemCmd++;
    	iDecControlCmd++;
    	iCurrentMsgId = iProxy->SendCommand(iProxyId, (OsclAny*) Msg);
    	iCmdSem.Wait();

    	return iSynCmdRet;
}


RET_STATUS DecProxyInterface::ProxyInit(DEC_HANDLETYPE hComponent)
{
	OsclAny *ptr = iMemoryPool->allocate(MAX_SIZE_OF_DEC_PROXY_MSG * sizeof(uint8));

	if (NULL == ptr)
	{
        	return Ret_ErrNoResources;
	}

    	InitMsg* Msg = OSCL_PLACEMENT_NEW(ptr, 
			InitMsg(hComponent));

	iMemCmd++;
    	iDecControlCmd++;
    	iCurrentMsgId = iProxy->SendCommand(iProxyId, (OsclAny*) Msg);
    	iCmdSem.Wait();

    	return iSynCmdRet;
}


RET_STATUS DecProxyInterface::ProxyPrepare(DEC_HANDLETYPE hComponent)
{
	OsclAny *ptr = iMemoryPool->allocate(MAX_SIZE_OF_DEC_PROXY_MSG * sizeof(uint8));

	if (NULL == ptr)
	{
        	return Ret_ErrNoResources;
	}

    	PrepareMsg* Msg = OSCL_PLACEMENT_NEW(ptr, 
			PrepareMsg(hComponent));

	iMemCmd++;
    	iDecControlCmd++;
    	iCurrentMsgId = iProxy->SendCommand(iProxyId, (OsclAny*) Msg);
    	iCmdSem.Wait();

    	return iSynCmdRet;
}

/*
	ProxyDecode is a Async command ,so the nCmid and pContext will be useful
*/
RET_STATUS DecProxyInterface::ProxyDecode(
	DEC_HANDLETYPE hComponent, DEC_UCHAR * pInputData, DEC_U32 nInputDataLen, DEC_BOOL bEos)
{
	OsclAny *ptr = iMemoryPool->allocate(MAX_SIZE_OF_DEC_PROXY_MSG * sizeof(uint8));

	if (NULL == ptr)
	{
        	return Ret_ErrNoResources;
	}

    	DecodeMsg* Msg = OSCL_PLACEMENT_NEW(ptr, 
			DecodeMsg(hComponent, pInputData , nInputDataLen , bEos));

	iMemCmd++;
    	iDecDataCmd++;
    	iCurrentMsgId = iProxy->SendCommand(iProxyId, (OsclAny*) Msg);

	/*
		Indicates this is a async command , caller should call 
	*/
    	return Ret_ErrWait;
}


RET_STATUS DecProxyInterface::ProxyDeinit(DEC_HANDLETYPE hComponent)
{
	OsclAny *ptr = iMemoryPool->allocate(MAX_SIZE_OF_DEC_PROXY_MSG * sizeof(uint8));

	if (NULL == ptr)
	{
        	return Ret_ErrNoResources;
	}

    	DeinitMsg* Msg = OSCL_PLACEMENT_NEW(ptr, 
			DeinitMsg(hComponent));

	iMemCmd++;
    	iDecControlCmd++;
    	iCurrentMsgId = iProxy->SendCommand(iProxyId, (OsclAny*) Msg, PROXY_PRIORITY_HIGH);
    	iCmdSem.Wait();

    	return iSynCmdRet;
}


RET_STATUS DecProxyInterface::ProxyGetHeaderInfo(
	DEC_HANDLETYPE hComponent, DEC_U32 * pFrameWidth, DEC_U32 * pFrameHeight, 
	DEC_U32 * pDisplayWidth, DEC_U32 * pDisplayHeight, DEC_U32 * pDecOutFormat)
{
	OsclAny *ptr = iMemoryPool->allocate(MAX_SIZE_OF_DEC_PROXY_MSG * sizeof(uint8));

	if (NULL == ptr)
	{
        	return Ret_ErrNoResources;
	}

    	GetHeaderInfoMsg* Msg = OSCL_PLACEMENT_NEW(ptr, 
			GetHeaderInfoMsg(hComponent, pFrameWidth , pFrameHeight, pDisplayWidth , pDisplayHeight ,
							pDecOutFormat));

	iMemCmd++;
    	iDecControlCmd++;
    	iCurrentMsgId = iProxy->SendCommand(iProxyId, (OsclAny*) Msg);
    	iCmdSem.Wait();

    	return iSynCmdRet;
}


RET_STATUS DecProxyInterface::OnePictureReadyCallback(DEC_UCHAR * pData, DEC_U32 nLen, DEC_PTR pParam)
{	
	OsclAny *ptr = iMemoryPool->allocate(MAX_SIZE_OF_DEC_PROXY_MSG * sizeof(uint8));

	if (NULL == ptr)
	{
		HWDEC_ERR("Cannot allocate notification messgae");
        	return Ret_ErrNoMemory;
	}

	CompleteNotification *notification = OSCL_PLACEMENT_NEW(ptr, 
			CompleteNotification( iCurrentHandlingMsgId , 
								PROXY_NOTIF_PICTURE_READY , Ret_ErrNone ,
								(DEC_U32)pParam , 
								(DEC_U32)pData , 
								(DEC_U32)nLen));

	iMemCmd++;

	iProxy->SendNotification( iProxyId , (OsclAny *)notification);

	return Ret_ErrNone;
}

/*
	This is running in the server threrad
*/
void DecProxyInterface::HandleCommand(
	TPVProxyMsgId aId, OsclAny * aMsg)
{
	proxyMsgType	cmd;
	DEC_BOOL syncCmd = true;
	DEC_HANDLETYPE component = NULL;
	RET_STATUS asyncRet;

	cmd = ((MsgBase *)aMsg)->getMsg();
	iCurrentHandlingMsgId = aId;

	switch (cmd)
    	{
    		case PROXY_CMD_GETHANDLE:
		{
			GetDecHandleMsg* msg = (GetDecHandleMsg *)aMsg;

			*(msg->pHandle) = 
				CreateDecoder( (DecodedDataProcessInterface *)(msg->pDp), 
								(DecProxyInterface *)(msg->pProxy), 
								msg->pAppContext , 
								msg->iCallback1, 
								msg->iCallback2 , 
								msg->inFmt , 
								msg->outFmt);

			if ( *(msg->pHandle) == NULL )
				iSynCmdRet = Ret_ErrGeneral;
			else
				iSynCmdRet = Ret_ErrNone;

			break;
		}

		case PROXY_CMD_FREEHANDLE:
		{
			FreeDecHandleMsg* msg = (FreeDecHandleMsg *)aMsg;

			DestructDecoder(msg->hComponent);
			iSynCmdRet = Ret_ErrNone;
			
			break;
		}
    	
		case PROXY_CMD_SETOUTPUTIMG:
        	{
            		SetOutputImgMsg* msg = (SetOutputImgMsg *)aMsg;

            		iSynCmdRet = ComponentSetOutputImage(msg->hComponent, 
												msg->nWidth , 
												msg->nHeight , 
												msg->nFormat);
			component = msg->hComponent;
			break;
		}
        	
		case PROXY_CMD_SETINPUTROTATION:
		{
			SetInputRotationMsg *msg = (SetInputRotationMsg *)aMsg;

			iSynCmdRet = ComponentSetInputRotation( msg->hComponent,
												msg->nRotation);
			component = msg->hComponent;
			break;
		}

		case PROXY_CMD_SETOUTPUTCROP:
		{
			SetOutputCropMsg *msg = (SetOutputCropMsg *)aMsg;

			iSynCmdRet = ComponentSetOutputCrop( msg->hComponent,
												msg->nTopLeftX,
												msg->nTopLeftY,
												msg->nWidth,
												msg->nHeight);
			component = msg->hComponent;
			break;
		}

		case PROXY_CMD_INIT:
		{
			InitMsg *msg = (InitMsg *)aMsg;

			iSynCmdRet = ComponentInit( msg->hComponent);
			component = msg->hComponent;
			break;
		}

		case PROXY_CMD_PREPARE:
		{
			PrepareMsg *msg = (PrepareMsg *)aMsg;

			iSynCmdRet = ComponentPrepare( msg->hComponent);

			if ( iSynCmdRet == Ret_ErrNone )
				iReadyToDecode = true;
			
			component = msg->hComponent;
			break;
		}

		case PROXY_CMD_DECODE:
		{
			DecodeMsg *msg = (DecodeMsg *)aMsg;

			if ( iReadyToDecode )
			{
				asyncRet = ComponentDecode( msg->hComponent , 
							msg->pData,
							msg->nDataLen,
							msg->bEos);
				
			}
			else
				asyncRet = Ret_ErrInvalidState;

			component = msg->hComponent;
			/*
				Decode is an async command
			*/
			syncCmd = false;
			break;
		}

		case PROXY_CMD_DEINIT:
		{
			DeinitMsg *msg = (DeinitMsg *)aMsg;

			iSynCmdRet = ComponentDeinit( msg->hComponent);
			iReadyToDecode = false;
			component = msg->hComponent;
			break;
		}

		case PROXY_CMD_GETHEADERINFO:
		{
			GetHeaderInfoMsg *msg = (GetHeaderInfoMsg *)aMsg;

			iSynCmdRet = ComponentGetHeaderInfo( msg->hComponent ,
											msg->pFrameW,
											msg->pFrameH,
											msg->pDisplayW,
											msg->pDisplayH,
											msg->pFormat);
			component = msg->hComponent;
			break;
		}

		default:
        	{
            		HWDEC_ERR("Unkown command!");
        	}
        	break;

	}

	if (syncCmd)
		iCmdSem.Signal();
	else
	{
		OsclAny *ptr = iMemoryPool->allocate(MAX_SIZE_OF_DEC_PROXY_MSG * sizeof(uint8));

		if (NULL == ptr)
		{
			HWDEC_ERR("Cannot allocate notification messgae");
        		return;
		}

    		CompleteNotification *notification = OSCL_PLACEMENT_NEW(ptr, 
			CompleteNotification(aId , cmd, asyncRet, (DEC_U32)component , 0 , 0));

		iMemCmd++;

		iProxy->SendNotification( iProxyId , (OsclAny *)notification);
	}

	iMemoryPool->deallocate((OsclAny*)aMsg);
	iMemCmd--;
}


/*
	This is running in the server thread
*/
void DecProxyInterface::CleanupNotification(
	TPVProxyMsgId aId, OsclAny * aMsg)
{
	iMemoryPool->deallocate((OsclAny*)aMsg);
	iMemCmd--;
}

/*
	This is running in the app thread
*/
void DecProxyInterface::HandleNotification(
	TPVProxyMsgId aId, OsclAny * aMsg)
{
	CompleteNotification *notification = (CompleteNotification *)aMsg;
	PVMFStatus status = PVMFSuccess;

//	HWDEC_MSG("HandleNoftification cmdid = %d", notification->nCmdId);

	if ( notification->nStatus != Ret_ErrNone)
		status = PVMFFailure;

	switch(notification->nMsg)
	{
		case PROXY_CMD_INIT:
		case PROXY_CMD_PREPARE:
		case PROXY_CMD_GETHANDLE:
		case PROXY_CMD_FREEHANDLE:
		case PROXY_CMD_DECODE:
		case PROXY_CMD_SETOUTPUTIMG:
		case PROXY_CMD_SETOUTPUTCROP:
		case PROXY_CMD_SETINPUTROTATION:
		case PROXY_CMD_DEINIT:
		case PROXY_CMD_GETHEADERINFO:

			HandleCommandDoneCallback( (DEC_HANDLETYPE)(notification->nParam1) , 
								status , 
								notification->nCmdId);
			break;

		case PROXY_NOTIF_PICTURE_READY:

			HandleOneFrameDecodedCallback( (DEC_HANDLETYPE)(notification->nParam1) , 
								status , 
								notification->nCmdId,
								notification->nParam2,
								notification->nParam3);
			break;

		default:
			HWDEC_ERR("Unkown message type");
			break;
			
	}
	
	iMemoryPool->deallocate((OsclAny *)aMsg);
	iMemCmd--;
}

/*
	This is running in the app thread
*/
void DecProxyInterface::CleanupCommand(
	TPVProxyMsgId aId, OsclAny * aMsg)
{
	iMemoryPool->deallocate((OsclAny*)aMsg);
	iMemCmd--;
}



