

#include <utils/Log.h>
#include <ui/ISurface.h>
#include <sys/timeb.h>
#include <media/PVPlayer.h>

#include "pvlogger.h"
#include "pv_mime_string_utils.h"
#include "oscl_snprintf.h"

#include "oscl_dll.h"

#include "hwdec_return_codes.h"
#include "hwdec_factory.h"
#include "hwdec_interface.h"
#include "video_surface_output.h"

#include "media_clock_converter.h"


// Define entry point for this DLL
OSCL_DLL_ENTRY_POINT_DEFAULT()

//The factory functions.
#include "oscl_mem.h"

///////////////////////////////////////////////////////////////////////////////////////

using namespace android;

#define LOG_TAG "VideoSurfaceOutput"

#define HW_RESIZE

#define SUPPORTED_COMPRESSED_FORMAT_NUM		7
#define DEFAULT_PENDING_CMD_NUM				5

#define VIDEO_EARLY_WINDOW	10
#define VIDEO_LATE_WINDOW		50

#define LCD_SCREEN_WIDTH		480
#define LCD_SCREEN_HEIGHT		272

/////////////////////////////////////////////////////////////////////////////////////////

extern RET_STATUS HWDEC_GetHandle(
	DEC_HANDLETYPE * pHandle, DecodedDataProcessInterface * dp, 
	DEC_PTR pAppData, DEC_BOOL useProxy, CommandDoneCallback callback1, NotificationCallback callback2, 
	DEC_INPUT_FORMAT inFmt, DEC_OUTPUT_FORMAT outFmt);


extern RET_STATUS HWDEC_FreeHandle(DEC_HANDLETYPE hComponent);


VideoSurfaceOutput::VideoSurfaceOutput() :
	iLCDWidth(LCD_SCREEN_WIDTH),
	iLCDHeight(LCD_SCREEN_HEIGHT),
	iSurfaceBufferWriteIndex(0),
	iSurfaceBufferReadIndex(0),
	iCurrentMediaInfo(NULL),
	iDecComponentHandler(NULL),
	iTrackConfig(NULL),
	iTrackConfigSize(0),
	iHdrDecoded(false),
	iMIOActivated(false)
{
	iCommandPendingQueue.reserve(DEFAULT_PENDING_CMD_NUM);

//	iSurfaceBufferSem = new OsclSemaphore();
//	iSurfaceBufferSem->Create(kBufferCount);

	OsclMemAllocator alloc;
	OsclAny *ptr = alloc.allocate(sizeof(VideoMIOActiveTiming));

	if (ptr) 
	{
		iActiveTiming = new(ptr)VideoMIOActiveTiming(this);
	}

	iRenderAO = new RenderAO( this , kBufferCount );

	if ( !iRenderAO )
		HWDEC_ERR("Canot allocate resource for RenderAO");
}


VideoSurfaceOutput::~VideoSurfaceOutput()
{
	if ( iTrackConfig )
		oscl_free(iTrackConfig);

	if ( iActiveTiming ) 
	{
		iActiveTiming->~VideoMIOActiveTiming();

		OsclMemAllocator alloc;
		alloc.deallocate(iActiveTiming);
	}

//	iSurfaceBufferSem->Close();
//	delete iSurfaceBufferSem;

	if (iRenderAO)
	{
		delete iRenderAO;
		iRenderAO = NULL;
	}
}


void VideoSurfaceOutput::CleanupCommandPendingQueue()
{
	while (!iCommandPendingQueue.empty())
	{
		switch( iCommandPendingQueue[0].iType )
		{
			case CMD_TYPE_DATA:
			{
				WriteResponse resp(PVMFSuccess, 
								iCommandPendingQueue[0].iPVMFCmdId, 
								iCommandPendingQueue[0].iContext ,
								iCommandPendingQueue[0].iTs);
				iWriteResponseQueue.push_back(resp);
				break;
			}
			
			case CMD_TYPE_CTRL:
			{
				CommandResponse resp( PVMFSuccess ,
									iCommandPendingQueue[0].iPVMFCmdId, 
									iCommandPendingQueue[0].iContext);
				QueueCommandResponse(resp);
				break;
			}

			default:
				break;
		}
		
        	iCommandPendingQueue.erase(&iCommandPendingQueue[0]);
	}
}


/*
	Proxy client function entry point , used by Notification complete AO , 
	this must run in the same thread as mio
*/
void VideoSurfaceOutput::HandleCommandDone(
	PVMFStatus status, TPVProxyMsgId cmdid)
{
	uint32 i;
	
	for( i = 0; i < iCommandPendingQueue.size(); i++ )
	{
		if ( iCommandPendingQueue[i].iProxyCmdId == cmdid )
		{
			switch ( iCommandPendingQueue[i].iType )
			{
				case CMD_TYPE_DATA:
				{
					/*
						Prepare to send buffer return command to connected port
					*/
					WriteResponse resp(status , 
									iCommandPendingQueue[i].iPVMFCmdId, 
									iCommandPendingQueue[i].iContext ,
									iCommandPendingQueue[i].iTs);
					iWriteResponseQueue.push_back(resp);
					RunIfNotReady();
				}
					break;
				
				case CMD_TYPE_CTRL:
				{
					CommandResponse resp( status ,
										iCommandPendingQueue[i].iPVMFCmdId, 
										iCommandPendingQueue[i].iContext);
					QueueCommandResponse(resp);
					RunIfNotReady();
					break;
				}
				
				case CMD_TYPE_HEAD:

					break;
					
				default:
					break;
			}

			iCommandPendingQueue.erase(&iCommandPendingQueue[i]);
			break;
		}
	}

}


void VideoSurfaceOutput::HandleOneFrameDecoded(
	PVMFStatus status, TPVProxyMsgId cmdid, DEC_U32 param1, DEC_U32 param2)
{
	/*
		Notify Render AO a frame is ready to be rendered
	*/
	uint32 i;

	/*
		First , we need to get the ts reserved
		We just fetch the information and we dont delete this pending command
		since it's HandleCommandDone's duty to delete it
	*/
	for( i = 0; i < iCommandPendingQueue.size(); i++ )
	{
		if ( iCommandPendingQueue[i].iProxyCmdId == cmdid )
		{
			iRenderAO->sendCommand(iCommandPendingQueue[i].iTs, (uint8 *)param1, (uint32)param2);
			break;
		}
	}

}


void VideoSurfaceOutput::BaseHandleCommandDone(
	DEC_HANDLETYPE hComponent, PVMFStatus status, TPVProxyMsgId cmdid)
{
	VideoSurfaceOutput *pMIO = 
		(VideoSurfaceOutput *)((DEC_COMPONENTTYPE *)hComponent)->applicationPrivateData;

	pMIO->HandleCommandDone( status, cmdid );
}


void VideoSurfaceOutput::BaseHandleOneFrameDecoded(
	DEC_HANDLETYPE hComponent, PVMFStatus status, TPVProxyMsgId cmdid, 
	DEC_U32 param1, DEC_U32 param2)
{
	VideoSurfaceOutput *pMIO = 
		(VideoSurfaceOutput *)((DEC_COMPONENTTYPE *)hComponent)->applicationPrivateData;

	pMIO->HandleOneFrameDecoded( status, cmdid , param1, param2);
}


void VideoSurfaceOutput::PictureResize(uint32 &resizeWidth , uint32 &resizeHeight)
{
	/*
		the w/h larger than LCDW/LCDH
	*/
	HWDEC_MSG("Doing the resize: w = %d , h = %d", iVideoWidth,iVideoHeight);
	if ( iVideoWidth*iLCDHeight > iVideoHeight*iLCDWidth )
	{
		resizeWidth = iLCDWidth;
		resizeHeight = ( iVideoHeight*resizeWidth)/iVideoWidth;
	}
	else
	{	resizeHeight = iLCDHeight;
		resizeWidth = ( iVideoWidth*resizeHeight)/iVideoHeight;
	}

	resizeHeight = (resizeHeight/8)*8;
	resizeWidth = (resizeWidth/8)*8;
	HWDEC_MSG("After the resize: w = %d , h = %d", resizeWidth, resizeHeight);
}


PVMFStatus VideoSurfaceOutput::DecoderInitAndPrepare()
{
	DEC_INPUT_FORMAT inFormat;
	RET_STATUS ret;

	if ( pv_mime_strcmp( iVideoFormatString.get_str() , PVMF_MIME_M4V ) == 0 ||
		pv_mime_strcmp( iVideoFormatString.get_cstr() , PVMF_MIME_H2632000) == 0 ||
		pv_mime_strcmp( iVideoFormatString.get_cstr() , PVMF_MIME_H2631998) == 0 ) 
				inFormat = IN_FMT_MPEG4;

	else if ( pv_mime_strcmp( iVideoFormatString.get_str() , PVMF_MIME_H264_VIDEO ) == 0 ||
			pv_mime_strcmp( iVideoFormatString.get_cstr() , PVMF_MIME_H264_VIDEO_MP4 ) == 0 ||
			pv_mime_strcmp( iVideoFormatString.get_cstr() , PVMF_MIME_H264_VIDEO_RAW ) == 0 ) 
				inFormat = IN_FMT_H264;
	else
	{
		/* Unsupported format , quit */
		inFormat = IN_FMT_INVALID;
		return PVMFFailure;
	}
			
	/* 
		Get a proper handler for vdec 
	*/
	ret = HWDEC_GetHandle( &iDecComponentHandler , 
							this , 
							this , 
							true, 
							BaseHandleCommandDone ,
							BaseHandleOneFrameDecoded,
							inFormat , 
							OUT_FMT_RGB565);

	if ( ret != Ret_ErrNone || !iDecComponentHandler )
	{
		HWDEC_ERR("HWDEC_GetHandler error");
		return PVMFFailure;
	}
	
	if ( HWDEC_Init( iDecComponentHandler ) != Ret_ErrNone )
	{
		HWDEC_ERR("HWDEC_Init error");
		HWDEC_FreeHandle( iDecComponentHandler);
		iDecComponentHandler = NULL;

		return PVMFFailure;
	}

	if ( HWDEC_SetOutputImg( iDecComponentHandler, iVideoWidth , iVideoHeight , V_OUT_RGB16 ) != Ret_ErrNone )
	{
		HWDEC_ERR("HWDEC_SetOutputImg error");
		HWDEC_Deinit( iDecComponentHandler );
		HWDEC_FreeHandle( iDecComponentHandler );
		iDecComponentHandler = NULL;

		return PVMFFailure;
	}
	
	if ( HWDEC_Prepare(iDecComponentHandler) != Ret_ErrNone )
	{
		HWDEC_ERR("HWDEC_Prepare error");
		HWDEC_Deinit( iDecComponentHandler );
		HWDEC_FreeHandle( iDecComponentHandler );
		iDecComponentHandler = NULL;

		return PVMFFailure;
	}

	iHdrDecoded = false;
	return PVMFSuccess;
}


PVMFCommandId VideoSurfaceOutput:: Init(const OsclAny* aContext)
{
	PVMFCommandId cmdid = iCommandCounter++;
	PVMFStatus status = PVMFFailure;

	switch(iState)
	{
		case STATE_LOGGED_ON:
		{
			iRenderAO->Init();
			status = DecoderInitAndPrepare();

			if ( status == PVMFSuccess )
			{
				iMIOActivated = true;
				iState = STATE_INITIALIZED;
			}
			break;
		}
		
		default:
			PVLOGGER_LOGMSG(PVLOGMSG_INST_LLDBG, iLogger, PVLOGMSG_STACK_TRACE, (0,"AndroidSurfaceOutput::Invalid State"));
			status = PVMFErrInvalidState;
			break;
	}
	
	CommandResponse resp(status,cmdid,aContext);
	QueueCommandResponse(resp);

	return cmdid;
}


PVMFCommandId VideoSurfaceOutput::Pause(const OsclAny* aContext)
{
	PVMFCommandId cmdid=iCommandCounter++;
	PVMFStatus status=PVMFFailure;

	switch(iState)
	{
		case STATE_STARTED:

			iState = STATE_PAUSED;
        		status = PVMFSuccess;

			break;

		default:
			status = PVMFErrInvalidState;
			break;
	}

	CommandResponse resp(status,cmdid,aContext);
	QueueCommandResponse(resp);

	return cmdid;
}


PVMFCommandId VideoSurfaceOutput::Stop(const OsclAny * aContext)
{
	iMIOActivated = false;
	
	if (iDecComponentHandler)
	{
		/*
			The render thread may be waitting for sth , we should make sure to wake it up
		*/
		HWDEC_ReturnBuffer(iDecComponentHandler);
		HWDEC_Deinit(iDecComponentHandler);
		HWDEC_FreeHandle(iDecComponentHandler);
		iDecComponentHandler = NULL;
	}

	iRenderAO->Exit();

	return AndroidSurfaceOutput::Stop(aContext);
}


PVMFCommandId VideoSurfaceOutput::Reset(const OsclAny* aContext)
{
	if (iDecComponentHandler)
	{
		HWDEC_Deinit(iDecComponentHandler);
		HWDEC_FreeHandle(iDecComponentHandler);
		iDecComponentHandler = NULL;
	}
	
	if ( iTrackConfig)
	{
		oscl_free(iTrackConfig);	
		iTrackConfig = NULL;
		iTrackConfigSize = 0;
	}

	PVMFCommandId cmdid = AndroidSurfaceOutput::Reset(aContext);
	
	return cmdid;
}


PVMFCommandId VideoSurfaceOutput::QueryUUID(const PvmfMimeString& aMimeType, 
                                        Oscl_Vector<PVUuid, OsclMemAllocator>& aUuids,
                                        bool aExactUuidsOnly, const OsclAny* aContext)
{
	OSCL_UNUSED_ARG(aMimeType);
	OSCL_UNUSED_ARG(aExactUuidsOnly);

	PVMFCommandId cmdid = iCommandCounter++;
	PVMFStatus status = PVMFFailure;
	int32 err ;

	OSCL_TRY(err, aUuids.push_back(PVMI_CAPABILITY_AND_CONFIG_PVUUID);
				if ( iActiveTiming )
				{
					aUuids.push_back(PvmiClockExtensionInterfaceUuid);
				}
			);

	if ( err == OsclErrNone )
		status= PVMFSuccess;

	CommandResponse resp(status,cmdid,aContext);	
	QueueCommandResponse(resp);

	return cmdid;
}


PVMFCommandId VideoSurfaceOutput::QueryInterface(const PVUuid& aUuid, PVInterface*& aInterfacePtr, const OsclAny* aContext)
{
	PVMFCommandId cmdid = iCommandCounter++;
	PVMFStatus status = PVMFFailure;
	
	if(aUuid == PVMI_CAPABILITY_AND_CONFIG_PVUUID)
	{
		PvmiCapabilityAndConfig* myInterface = OSCL_STATIC_CAST(PvmiCapabilityAndConfig*,this);
		aInterfacePtr = OSCL_STATIC_CAST(PVInterface*, myInterface);
		status = PVMFSuccess;
	}
	else if( aUuid ==  PvmiClockExtensionInterfaceUuid)
	{
		if ( iActiveTiming->queryInterface( aUuid, aInterfacePtr ) )
		{
			HWDEC_MSG("QueryInterface clock extension available");
			status = PVMFSuccess;
		}
	}

	CommandResponse resp(status,cmdid,aContext);
	QueueCommandResponse(resp);

	return cmdid;
}



PVMFStatus VideoSurfaceOutput::getParametersSync(PvmiMIOSession aSession, PvmiKeyType aIdentifier,
                                              PvmiKvp*& aParameters, int& num_parameter_elements,
                                              PvmiCapabilityContext aContext)
{
	OSCL_UNUSED_ARG(aSession);
	OSCL_UNUSED_ARG(aContext);

	aParameters = NULL;
	num_parameter_elements = 0;

	/*
		This is a query for the list of supported formats.
	*/
	if( pv_mime_strcmp(aIdentifier, INPUT_FORMATS_CAP_QUERY) == 0 )
	{
		uint32 count;
		uint32 i;

		/*
			Hardware decoder dont support REAL format here.
		*/
		count = SUPPORTED_COMPRESSED_FORMAT_NUM;
		i = num_parameter_elements;
		aParameters = (PvmiKvp*)oscl_malloc(sizeof(PvmiKvp)*count );

		if (aParameters == NULL) 
		{
			HWDEC_ERR("Not enought memory for parameters");
			return PVMFErrNoMemory;
		}
		
		aParameters[num_parameter_elements++].value.pChar_value = (char *)PVMF_MIME_M4V;
		aParameters[num_parameter_elements++].value.pChar_value = (char *)PVMF_MIME_H264_VIDEO;
		aParameters[num_parameter_elements++].value.pChar_value = (char *)PVMF_MIME_H264_VIDEO_MP4;
		aParameters[num_parameter_elements++].value.pChar_value = (char *)PVMF_MIME_H264_VIDEO_RAW;
		aParameters[num_parameter_elements++].value.pChar_value = (char *)PVMF_MIME_H2631998;
		aParameters[num_parameter_elements++].value.pChar_value = (char *)PVMF_MIME_H2632000;
		aParameters[num_parameter_elements++].value.pChar_value = (char *)PVMF_MIME_WMV;

		while ( i < count )
		{
			aParameters[i].key = (PvmiKeyType)oscl_malloc(oscl_strlen(MOUT_VIDEO_FORMAT_KEY) + 1);

			if (!aParameters[i].key)
			{
				HWDEC_ERR("Not enought memory for parameters key");
				return PVMFErrNoMemory;
			}
			
			oscl_strncpy(aParameters[i++].key, MOUT_VIDEO_FORMAT_KEY, oscl_strlen(MOUT_VIDEO_FORMAT_KEY) + 1);
		}

		return PVMFSuccess;
	}

	/*unrecognized key.*/
	return PVMFFailure;
}


void VideoSurfaceOutput::setParametersSync(PvmiMIOSession aSession, PvmiKvp* aParameters,
                                        int num_elements, PvmiKvp * & aRet_kvp)
{
	OSCL_UNUSED_ARG(aSession);

	aRet_kvp = NULL;
  
	for ( int32 i = 0 ; i < num_elements ; i++ )
	{
		/*
			Set input video data format
		*/
		if (pv_mime_strcmp(aParameters[i].key, MOUT_VIDEO_FORMAT_KEY) == 0)
		{
			iVideoFormatString = aParameters[i].value.pChar_value;
			iVideoFormat = iVideoFormatString.get_str();
			HWDEC_MSG("Video Format Key, Value %s", iVideoFormatString.get_str() );
		}
		/*
			This parameter is very important , 
			TrackConfig contain the video header specific information for different video format
			This will be the first packet that passed to hardware decoder.
		*/
		else if ( pv_mime_strcmp(aParameters[i].key , PVMF_FORMAT_SPECIFIC_INFO_KEY ) == 0 )
		{
			if ( iTrackConfigSize > 0 && iTrackConfig )
			{
				oscl_free( iTrackConfig);
				iTrackConfigSize = 0;
			}
		
			iTrackConfigSize = aParameters[i].capacity;
                	iTrackConfig = (uint8*)(OSCL_MALLOC(sizeof(uint8) * iTrackConfigSize));
                	oscl_memcpy(iTrackConfig, aParameters[i].value.key_specific_value, iTrackConfigSize);	
			iVideoParameterFlags |= VIDEO_SUBFORMAT_VALID;

			HWDEC_MSG("iTrackConfigSize = %d" , iTrackConfigSize);
		}
		/*
			Video width
		*/
		else if (pv_mime_strcmp(aParameters[i].key, MOUT_VIDEO_WIDTH_KEY) == 0)
		{
			iVideoWidth = (int32)aParameters[i].value.uint32_value;
			iVideoParameterFlags |= VIDEO_WIDTH_VALID;
                	HWDEC_MSG( "Video Width Key, Value %d",iVideoWidth);
		}
		/*
			Video Height
		*/
		else if (pv_mime_strcmp(aParameters[i].key, MOUT_VIDEO_HEIGHT_KEY) == 0)
		{
			iVideoHeight=(int32)aParameters[i].value.uint32_value;
			iVideoParameterFlags |= VIDEO_HEIGHT_VALID;
			HWDEC_MSG("Video Height Key, Value %d",iVideoHeight);
        	}
		/*
			Video display width
		*/
		else if (pv_mime_strcmp(aParameters[i].key, MOUT_VIDEO_DISPLAY_HEIGHT_KEY) == 0)
		{
			iVideoDisplayHeight=(int32)aParameters[i].value.uint32_value;
			iVideoParameterFlags |= DISPLAY_HEIGHT_VALID;
                	HWDEC_MSG("Video Display Height Key, Value %d",iVideoDisplayHeight);
		}
		/*
			Video display height
		*/
		else if (pv_mime_strcmp(aParameters[i].key, MOUT_VIDEO_DISPLAY_WIDTH_KEY) == 0)
		{
			iVideoDisplayWidth = (int32)aParameters[i].value.uint32_value;
			iVideoParameterFlags |= DISPLAY_WIDTH_VALID;
			HWDEC_MSG("Video Display Width Key, Value %d",iVideoDisplayWidth);
		}
		/*
			Video subformat 
		*/
		else if (pv_mime_strcmp(aParameters[i].key, MOUT_VIDEO_SUBFORMAT_KEY) == 0)
		{
			iVideoSubFormat=aParameters[i].value.pChar_value;
            
		iVideoParameterFlags |= VIDEO_SUBFORMAT_VALID;
			HWDEC_MSG("Video SubFormat Key, Value %s",iVideoSubFormat.getMIMEStrPtr());
		}
		else
		{
			/*
				if we get here the key is unrecognized.
			*/
			HWDEC_MSG("unrecognized key = %s", aParameters[i].key);

			/*
				set the return value to indicate the unrecognized key and return.
			*/
			aRet_kvp = &aParameters[i];
			return;
		}
	}

	uint32 mycache = iVideoParameterFlags ;

	/*
		if initialization is complete, update the app display info
	*/
	if( checkVideoParameterFlags() )
	{
#ifdef HW_RESIZE
		uint32 resizeWidth, resizeHeight;

		PictureResize( resizeWidth, resizeHeight);

		iVideoWidth = resizeWidth;
		iVideoHeight = resizeHeight;
		iVideoDisplayWidth = resizeWidth;
		iVideoDisplayHeight = resizeHeight;
#endif		
		initCheck();
	}
	
	iVideoParameterFlags = mycache;

	/*
		when all necessary parameters are received, send 
	 	PVMFMIOConfigurationComplete event to observer
	*/
	if(!iIsMIOConfigured && checkVideoParameterFlags() )
	{	
		iIsMIOConfigured = true;

		if(iObserver)
        	{
			iObserver->ReportInfoEvent(PVMFMIOConfigurationComplete);
		}
	}
}


PVMFStatus VideoSurfaceOutput::verifyParametersSync (PvmiMIOSession aSession, PvmiKvp* aParameters, int num_elements)
{
	OSCL_UNUSED_ARG(aSession);

	/*
		Go through each parameter
	*/
	for ( int32 i = 0;  i < num_elements; i++ )
	{
		char* compstr = NULL;

		pv_mime_string_extract_type(0, aParameters[i].key, compstr);

		/*
			Verify format type
		*/
		if (pv_mime_strcmp(compstr, _STRLIT_CHAR("x-pvmf/media/format-type")) == 0)
		{
			if ( pv_mime_strcmp( aParameters[i].value.pChar_value, PVMF_MIME_M4V ) == 0 || 
				pv_mime_strcmp( aParameters[i].value.pChar_value, PVMF_MIME_H2631998 ) == 0 ||
				pv_mime_strcmp( aParameters[i].value.pChar_value, PVMF_MIME_H2632000 ) == 0 ||
				pv_mime_strcmp( aParameters[i].value.pChar_value, PVMF_MIME_H264_VIDEO ) == 0 ||
				pv_mime_strcmp( aParameters[i].value.pChar_value, PVMF_MIME_H264_VIDEO_MP4 ) == 0 ||
				pv_mime_strcmp( aParameters[i].value.pChar_value, PVMF_MIME_H264_VIDEO_RAW ) == 0 ||
				pv_mime_strcmp( aParameters[i].value.pChar_value, PVMF_MIME_WMV ) == 0)
			{
				return PVMFSuccess;
			}
			else
			{
                		return PVMFErrNotSupported;
			}
		}

		/*
			Veryify Format Specific Info
		*/
		else if ( pv_mime_strcmp( compstr , _STRLIT_CHAR("x-pvmf/media/format_specific_info")) == 0 )
			return PVMFSuccess;
		
	}

	return PVMFSuccess;
}


PVMFCommandId VideoSurfaceOutput::writeAsync(uint8 aFormatType, int32 aFormatIndex, uint8* aData, uint32 aDataLen,
                                        const PvmiMediaXferHeader& data_header_info, OsclAny* aContext)
{
	/*
    		Do a leave if MIO is not configured except when it is an EOS
	*/
	if (!iIsMIOConfigured
		&&!((PVMI_MEDIAXFER_FMT_TYPE_NOTIFICATION == aFormatType)
			&& (PVMI_MEDIAXFER_FMT_INDEX_END_OF_STREAM == aFormatIndex)))
	{
		HWDEC_ERR("data is pumped in before MIO is configured");
		OSCL_LEAVE(OsclErrInvalidState);
		return -1;
	}

	uint32 aSeqNum = data_header_info.seq_num;
	PVMFTimestamp aTimestamp = data_header_info.timestamp;
	uint32 flags = data_header_info.flags;

	if (aSeqNum < 6)
	{
		PVLOGGER_LOGMSG(PVLOGMSG_INST_LLDBG, iLogger, PVLOGMSG_STACK_TRACE,
            	(0,"AndroidSurfaceOutput::writeAsync() seqnum %d ts %d context %d",aSeqNum,aTimestamp,aContext));

		PVLOGGER_LOGMSG(PVLOGMSG_INST_LLDBG, iLogger, PVLOGMSG_STACK_TRACE,
		(0,"AndroidSurfaceOutput::writeAsync() Format Type %d Format Index %d length %d",aFormatType,aFormatIndex,aDataLen));
    	}
	
	PVMFCommandId cmdid = iCommandCounter++;
	PVMFStatus status = PVMFFailure;

	switch(aFormatType)
	{
		case PVMI_MEDIAXFER_FMT_TYPE_COMMAND :
			PVLOGGER_LOGMSG(PVLOGMSG_INST_LLDBG, iLogger, PVLOGMSG_STACK_TRACE,
				(0,"AndroidSurfaceOutput::writeAsync() called with Command info."));

        		status = PVMFSuccess;
        		break;

    		case PVMI_MEDIAXFER_FMT_TYPE_NOTIFICATION :
        		PVLOGGER_LOGMSG(PVLOGMSG_INST_LLDBG, iLogger, PVLOGMSG_STACK_TRACE,
            			(0,"AndroidSurfaceOutput::writeAsync() called with Notification info."));

			switch(aFormatIndex)
        		{
        			case PVMI_MEDIAXFER_FMT_INDEX_END_OF_STREAM:
					iEosReceived = true;
            				break;

				default:
            				break;
        		}
        		//ignore
        		status = PVMFSuccess;
        		break;

		case PVMI_MEDIAXFER_FMT_TYPE_DATA :

			switch(aFormatIndex)
        		{
				case PVMI_MEDIAXFER_FMT_INDEX_FMT_SPECIFIC_INFO:

					//format-specific info contains codec headers.
            				PVLOGGER_LOGMSG(PVLOGMSG_INST_LLDBG, iLogger, PVLOGMSG_STACK_TRACE,
                				(0,"AndroidSurfaceOutput::writeAsync() called with format-specific info."));

            				if (iState < STATE_INITIALIZED)
            				{
                				PVLOGGER_LOGMSG(PVLOGMSG_INST_REL, iLogger, PVLOGMSG_ERR,
                    					(0,"AndroidSurfaceOutput::writeAsync: Error - Invalid state"));
                				status = PVMFErrInvalidState;
            				}
            				else
            				{
                				status = PVMFSuccess;
            				}

            				break;

        			case PVMI_MEDIAXFER_FMT_INDEX_DATA:
				{
					if ( !iDecComponentHandler)
					{
						HWDEC_ERR("Invalid state , drop frame!###########");
						status = PVMFFailure;
						break;
					}
					
                			status = writeFrameBuf(aData, aDataLen, data_header_info);

					/*
						Hanlde async command here
					*/
					if ( status == PVMFPending )
					{
						TPVProxyMsgId proxyId;

						proxyId = HWDEC_GetCommandId(iDecComponentHandler);
						CommandPending pendingCmd(proxyId , cmdid , aContext , aTimestamp , CMD_TYPE_DATA);
							iCommandPendingQueue.push_back(pendingCmd);

						return cmdid;
					}

                			PVLOGGER_LOGMSG(PVLOGMSG_INST_REL, iLogger, PVLOGMSG_ERR,
                   				(0,"AndroidSurfaceOutput::writeAsync: Playback Progress - frame %d",iFrameNumber++));
            			}
            			break;

        			default:
            				PVLOGGER_LOGMSG(PVLOGMSG_INST_REL, iLogger, PVLOGMSG_ERR,
                				(0,"AndroidSurfaceOutput::writeAsync: Error - unrecognized format index"));
            				status = PVMFFailure;
            				break;
        		}
        		break;

		default:
        		PVLOGGER_LOGMSG(PVLOGMSG_INST_REL, iLogger, PVLOGMSG_ERR,
            			(0,"AndroidSurfaceOutput::writeAsync: Error - unrecognized format type"));
        		status = PVMFFailure;
        		break;
	}

    	/*
    		Schedule asynchronous response
    	*/
    	
	WriteResponse resp(status,cmdid,aContext,aTimestamp);
    	iWriteResponseQueue.push_back(resp);
    	RunIfNotReady();

    	return cmdid;
}



OSCL_EXPORT_REF PVMFStatus VideoSurfaceOutput::writeFrameBuf(uint8* aData, uint32 aDataLen, 
													const PvmiMediaXferHeader& data_header_info)
{
	PVMFStatus status;

//	HWDEC_MSG("writeFrameBuf idx=%d ts=%d", data_header_info.seq_num , data_header_info.timestamp);

	if ( !iHdrDecoded )
	{
		if ( pv_mime_strcmp( iVideoFormatString.get_str() , PVMF_MIME_M4V ) == 0 ||
				pv_mime_strcmp( iVideoFormatString.get_cstr() , PVMF_MIME_H2632000) == 0 ||
				  pv_mime_strcmp( iVideoFormatString.get_cstr() , PVMF_MIME_H2631998) == 0 ) 
		{
			HWDEC_Decode(iDecComponentHandler, iTrackConfig , iTrackConfigSize , false );
		}
		else if ( pv_mime_strcmp( iVideoFormatString.get_str() , PVMF_MIME_H264_VIDEO ) == 0 ||
				pv_mime_strcmp( iVideoFormatString.get_cstr() , PVMF_MIME_H264_VIDEO_MP4 ) == 0 ||
				  pv_mime_strcmp( iVideoFormatString.get_cstr() , PVMF_MIME_H264_VIDEO_RAW ) == 0 ) 
		{
			uint16 length , size = 0;
			uint8 *pNal = iTrackConfig;

			do
           	 	{
                		length = (uint16)( pNal[1] << 8) | pNal[0];
                		size += (length + 2);

				if (size > iTrackConfigSize )
                    			break;
                		pNal += 2;

				HWDEC_Decode(iDecComponentHandler, pNal , (uint32)length , false );
                		pNal += length;

            		}while (size < iTrackConfigSize );
			
		}

		iHdrDecoded = true;
	}

	RET_STATUS ret;

	/*
		This must be set before decoding , since the OnePictureReadyCallback will use it
		But it doesnt matter when using proxy interface , this step is not necessary
	*/
	iCurrentMediaInfo = (PvmiMediaXferHeader *)&data_header_info;
	ret = HWDEC_Decode( iDecComponentHandler, aData , aDataLen , false );

	if ( ret == Ret_ErrWait )
		/*
			Ret_ErrWait indicates this is a async operation , we should do something
		*/
		return PVMFPending;

	if ( ret != Ret_ErrNone)
		return PVMFFailure;

	return PVMFSuccess;
}

/*
	This maybe running in another thread, so protection is very important
*/
RET_STATUS VideoSurfaceOutput::OnePictureReadyCallback(uint8 * data, uint32 len, void * param)
{
	OSCL_UNUSED_ARG(param);

	HWDEC_MSG("Local->OnePictureReadyCallback");
	
	if ( iRenderAO )
		iRenderAO->sendCommand(iCurrentMediaInfo->timestamp , data, len );

	return Ret_ErrNone;
}

/*
	RenderProcInterface implementation
*/
DecMediaTimeStatus VideoSurfaceOutput::RenderCheckTimestamp(int32 early, int32 late, PVMFTimestamp ts, uint32 & delta)
{
	if ( iActiveTiming )
		return iActiveTiming->CheckTimestamp( early, late, ts, delta);
	else
		return DEC_MEDIA_ERROR;
}

void VideoSurfaceOutput::RenderDataCallback( uint8 *data , uint32 len, OsclAny * context )
{
	OSCL_UNUSED_ARG(context);

	uint8 *postData;
	uint32 postLen;



	if ((mSurface != NULL) && (mBufferHeap.heap != NULL))
	{
		if ( !iMIOActivated )
		{
			HWDEC_ERR("MIO not activated");
			return ;
		}
		
		postData = static_cast<uint8*>(mBufferHeap.heap->base()) + mFrameBuffers[mFrameBufferIndex];
		postLen = len;
		oscl_memcpy( postData , data , postLen );
		mSurface->postBuffer(mFrameBuffers[mFrameBufferIndex]);

		if ( ++mFrameBufferIndex == kBufferCount )
			mFrameBufferIndex = 0;
	}
}


void VideoSurfaceOutput::RenderOverNotificationCallback(OsclAny * context)
{
	OSCL_UNUSED_ARG(context);

	HWDEC_ReturnBuffer(iDecComponentHandler);
}

PVMFStatus VideoSurfaceOutput::RenderTimeoutCallback( uint32 &callbackID, uint32 timeout, TCallback callback, OsclAny * context)
{
	if ( iActiveTiming )
		return iActiveTiming->SetTimeout( callbackID, timeout, callback, context);
	else
		return PVMFFailure;
}

PVMFStatus VideoSurfaceOutput::RenderCancelTimeoutCallback(uint32 callbackID)
{
	if ( iActiveTiming )
		return iActiveTiming->CancelTimeout(callbackID);
	else
		return PVMFFailure;
}

PVMFStatus VideoSurfaceOutput::ClockStateChangedCallback( 
	PVMFMediaClock::PVMFMediaClockState oldState, PVMFMediaClock::PVMFMediaClockState newState)
{
	switch (newState) 
	{
		case PVMFMediaClock::STOPPED:
			HWDEC_MSG("A/V clock stopped");					
			break;
					
		case PVMFMediaClock::RUNNING:
			HWDEC_MSG("A/V clock running");
			iRenderAO->Start();
			break;
					
		case PVMFMediaClock::PAUSED:
			HWDEC_MSG("A/V clock paused");
			iRenderAO->Pause();
			break;

		default:
			HWDEC_MSG("Wrong clock state!");
			break;
	}

	return PVMFSuccess;
}

/////////////////////////////////RenderAO////////////////////////////////////////////////////
#undef LOG_TAG
#define LOG_TAG "RendAO"

void RenderAO::Init()
{
	AddToScheduler();
	PendForExec();
	iState = Render_initialized;
}


void RenderAO::Start()
{
	if ( iState == Render_initialized )
		PendComplete(OSCL_REQUEST_ERR_NONE);
	
	iState = Render_running;
}

void RenderAO::Pause()
{
	/* Not need to do anything */
}

void RenderAO::Exit()
{
	if ( iCurrentTimeoutID != 0 && iRenderIf)
		iRenderIf->RenderCancelTimeoutCallback( iCurrentTimeoutID );

	iCurrentTimeoutID = 0;
	iProcessIncomingRequest = false;
	
	RemoveFromScheduler();
	CleanupRenderCommandQueue();
}

PVMFStatus RenderAO::sendCommand( PVMFTimestamp ts, uint8 * data, uint32 size)
{
	RenderCommand command(ts,data,size);
	iRenderCommandQueue.push_back(command);

	RunIfNotReady();

	return PVMFSuccess;
}

void RenderAO::CleanupRenderCommandQueue()
{
	while( !iRenderCommandQueue.empty() )
	{
		iRenderCommandQueue.erase(&iRenderCommandQueue[0]);
	}
}


void RenderAO::Run()
{
	DecMediaTimeStatus status;
	uint32 delta;
	bool NoPending;

	if ( !iRenderIf )
	{
		HWDEC_ERR("No render interface has been set yet!");
		return ;
	}

	while( !iRenderCommandQueue.empty())
	{
		NoPending = true;
		
		status = iRenderIf->RenderCheckTimestamp(VIDEO_EARLY_WINDOW, 
											VIDEO_LATE_WINDOW, 
											iRenderCommandQueue[0].iTs , 
											delta);

		switch(status)
		{
			case DEC_MEDIA_ONTIME:
//				HWDEC_MSG("Ontime");
				iRenderIf->RenderDataCallback( iRenderCommandQueue[0].pData, 
											iRenderCommandQueue[0].iLen);
	
				break;

			case DEC_MEDIA_EARLY:
//				HWDEC_MSG("Early");
				NoPending = false;
				break;
				
			case DEC_MEDIA_LATE:
//				HWDEC_MSG("Late");
				break;
				
			case DEC_MEDIA_ERROR:

				break;
				
			default:
				break;
		}

		if ( NoPending )
		{
			iRenderCommandQueue.erase(&iRenderCommandQueue[0]);

			/*
				We notify the decoder thread that a free surface buffer is available
			*/
			if ( iRenderIf )
				iRenderIf->RenderOverNotificationCallback();
		}
		/*
			It's early to deliver the data , so we make a deffered run
		*/
		else
		{
			iCurrentTimeoutID = 
				iRenderIf->RenderTimeoutCallback( iCurrentTimeoutID, delta , RenderAO::TimeoutCallback, this);
			iProcessIncomingRequest = false;
			PendForExec();

			break;
		}
	}
}



void RenderAO::TimeoutCallback(OsclAny *param)
{
	RenderAO *selfAO = (RenderAO *)param;

	if ( !selfAO)
	{
		HWDEC_ERR("Big error occured here!selfAO should never be null");
		return ;
	}

	selfAO->PendComplete(OSCL_REQUEST_ERR_NONE);
	selfAO->iProcessIncomingRequest = true;
	selfAO->iCurrentTimeoutID = 0;
}


//////////////////////////////////VideoMIOActiveTiming//////////////////////////////////////////

#undef LOG_TAG
#define LOG_TAG "VideoMIOActiveTiming"

void VideoMIOActiveTiming::cleanupRequestQueue()
{
	while( !iTimeoutRequestQueue.empty() )
	{
		iTimeoutRequestQueue.erase(&iTimeoutRequestQueue[0]);
	}
}

DecMediaTimeStatus VideoMIOActiveTiming::CheckTimestamp( int32 early , int32 late , PVMFTimestamp ts, uint32& aDelta )
{
	PVMFTimestamp aTimeStamp = ts;
	aDelta = 0;

	if (iClock != NULL)
	{
		uint32 clock_msec32;
		bool overflowFlag = false;

		iClock->GetCurrentTime32(clock_msec32, overflowFlag, PVMF_MEDIA_CLOCK_MSEC);

		uint32 clock_adjforearlymargin = clock_msec32 + early;
		uint32 ts_adjforlatemargin = aTimeStamp + late;

		if ((clock_adjforearlymargin - aTimeStamp) > WRAP_THRESHOLD)
		{
			aDelta = (aTimeStamp - clock_msec32);
			aDelta -= early;
//			HWDEC_MSG("EARLY ts=%d clock=%d delta=%d", ts , clock_msec32,aDelta );
			return DEC_MEDIA_EARLY;
		}
        	else if ((ts_adjforlatemargin - clock_msec32) > WRAP_THRESHOLD)
		{
			aDelta = (clock_msec32 - aTimeStamp);
			HWDEC_MSG("LATE ts=%d clock=%d delta=%d", ts , clock_msec32,aDelta);
			return DEC_MEDIA_LATE;
		}
		else
		{
//			HWDEC_MSG("ONTIME ts=%d clock=%d delta=%d", ts , clock_msec32,aDelta );
			return DEC_MEDIA_ONTIME;
		}
	}
	else
	{
		return DEC_MEDIA_ERROR;
	}
}


PVMFStatus VideoMIOActiveTiming::SetTimeout(
	uint32 & callbackID, uint32 timeout, TCallback callback, OsclAny * context)
{
	PVMFStatus status;

	status = iClockNotificationsInf->SetCallbackDeltaTime( timeout , 0, this, false, NULL, callbackID);

	if ( status != PVMFSuccess )
	{
		HWDEC_ERR("Cannot set callback delta time !");
		return PVMFFailure;
	}

	TimeoutRequest req(callbackID, callback, context);
	iTimeoutRequestQueue.push_back(req);

	return PVMFSuccess;
}


PVMFStatus VideoMIOActiveTiming::CancelTimeout(uint32 callbackID)
{
	if ( iClockNotificationsInf )
		return iClockNotificationsInf->CancelCallback( callbackID, false);
	else
		return PVMFFailure;
}


void VideoMIOActiveTiming::ProcessCallBack(
	uint32 callBackID,  PVTimeComparisonUtils::MediaTimeStatus aTimerAccuracy, 
	uint32 aDelta, const OsclAny * aContextData, PVMFStatus aStatus)
{
	OSCL_UNUSED_ARG(aTimerAccuracy);
	OSCL_UNUSED_ARG(aDelta);
	OSCL_UNUSED_ARG(aContextData);

	int i;

	for ( i = 0; i < iTimeoutRequestQueue.size(); i++ )
	{
		if ( iTimeoutRequestQueue[i].iCallbackID == callBackID )
		{
			iTimeoutRequestQueue[i].iCallback(iTimeoutRequestQueue[i].iContext);
			iTimeoutRequestQueue.erase(&iTimeoutRequestQueue[i]);

			break;
		}
	}
}


PVMFStatus VideoMIOActiveTiming::SetClock(PVMFMediaClock * clockVal)
{
	iClock = clockVal;

	if (iClock)
	{
		PVMFStatus ret = iClock->ConstructMediaClockNotificationsInterface(iClockNotificationsInf,*this);

		if( iClockNotificationsInf && ( PVMFSuccess == ret) )
        	{
            		iClockNotificationsInf->SetClockStateObserver(*this);
        	}
        	else
        	{
            		HWDEC_ERR("Construct clockNotificationInf error");
			return PVMFFailure;
        	}
	}
	
	return PVMFSuccess;
}


void VideoMIOActiveTiming::ClockStateUpdated()
{
	if (iClock) 
	{
		PVMFMediaClock::PVMFMediaClockState newClockState = iClock->GetState();

		if (newClockState != iClockState) 
		{
			if ( iClock )
				iClockStateCallbackInf->ClockStateChangedCallback( iClockState , newClockState );
			
			iClockState = newClockState;
		}
	}
}


bool VideoMIOActiveTiming::queryInterface(const PVUuid & uuid, PVInterface * & iface)
{
	iface = NULL;

	if (uuid == PvmiClockExtensionInterfaceUuid) 
	{
		PvmiClockExtensionInterface* myInterface = OSCL_STATIC_CAST(PvmiClockExtensionInterface*, this);
		iface = OSCL_STATIC_CAST(PVInterface*, myInterface);

		return true;
	}
	
	return false;
}


void VideoMIOActiveTiming::NotificationsInterfaceDestroyed()
{
	iClockNotificationsInf = NULL;
}

//////////////////////////////// Entry Point ////////////////////////////////////////////////////

extern "C" AndroidSurfaceOutput* createVideoMio()
{
	return new VideoSurfaceOutput();
}


