/*
** Copyright (C) 2009.11 Embest For Pxa310-Android1.6 camera By XuHaibing- http://www.embedinfo.com/
** Copyright (C) 2009 0xlab.org - http://0xlab.org/
** Copyright 2008, The Android Open Source Project
**
** Licensed under the Apache License, Version 2.0 (the "License");
** you may not use this file except in compliance with the License.
** You may obtain a copy of the License at
**
**     http://www.apache.org/licenses/LICENSE-2.0
**
** Unless required by applicable law or agreed to in writing, software
** distributed under the License is distributed on an "AS IS" BASIS,
** WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
** See the License for the specific language governing permissions and
** limitations under the License.
*/

#ifndef ANDROID_HARDWARE_CAMERA_HARDWARE_H
#define ANDROID_HARDWARE_CAMERA_HARDWARE_H

#include <utils/threads.h>
#include <ui/CameraHardwareInterface.h>
#include <binder/MemoryBase.h>
#include <binder/MemoryHeapBase.h>
#include <utils/threads.h>

#include <jpeglib.h>
#include "V4L2Camera.h"

#define PIXEL_FORMAT        V4L2_PIX_FMT_YUYV

namespace android {

class CameraHardware : public CameraHardwareInterface {
public:
	virtual sp<IMemoryHeap> getPreviewHeap() const;	 
	virtual sp<IMemoryHeap> getRawHeap() const;	
    
	virtual void        setCallbacks(notify_callback notify_cb,
                                     data_callback data_cb,
                                     data_callback_timestamp data_cb_timestamp,
                                     void* user);
                                     
	virtual void        enableMsgType(int32_t msgType);
	virtual void        disableMsgType(int32_t msgType);
	virtual bool        msgTypeEnabled(int32_t msgType);

	virtual status_t    startPreview();
	virtual void         stopPreview();
	virtual bool         previewEnabled();

	virtual status_t    startRecording();
	virtual void         stopRecording();
	virtual bool         recordingEnabled();
	virtual void         releaseRecordingFrame(const sp<IMemory>& mem);

	virtual status_t    autoFocus();
	virtual status_t    cancelAutoFocus();
	virtual status_t    takePicture();
	virtual status_t    cancelPicture();
	virtual status_t    dump(int fd, const Vector<String16>& args) const;
	virtual status_t    setParameters(const CameraParameters& params);
	virtual CameraParameters  getParameters() const;
	virtual status_t                   sendCommand(int32_t command, int32_t arg1,
                                    int32_t arg2);
	virtual void release();
	int 		previewThread();
	static sp<CameraHardwareInterface> createInstance();
    
private:
				CameraHardware();
	virtual            ~CameraHardware();

	static wp<CameraHardwareInterface> singleton;
 
 	int				mPreview_Width; 
	int 				mPreview_Height;
	int				mPicture_Width;
	int				mPicture_Height;
	int				mDebugFps;
	size_t			mPreviewFrameSize;

	class PreviewThread : public Thread {
		CameraHardware* mHardware;
		public:
			PreviewThread(CameraHardware* hw)
				: Thread(false), mHardware(hw) { }
			virtual void onFirstRef() {
				run("CameraPreviewThread", PRIORITY_URGENT_DISPLAY);
			}
			virtual bool threadLoop() {	
			mHardware->previewThread();
			// loop until we need to quit
			return true;
		}
	};

	void 	initDefaultParameters();
	void 	initHeapLocked();
    
    	static int 	beginPreviewThread(void *arg); 
    	static int 	beginAutoFocusThread(void *cookie);
    	int 		autoFocusThread();

    	static int 	beginPictureThread(void *cookie);
    	int 		pictureThread();

    	mutable Mutex       		mLock;
	mutable Mutex       		camLock;

    	CameraParameters    		mParameters;

    	sp<MemoryHeapBase>  	mRawHeap;  // format: 565
    	sp<MemoryBase>      		mRawBuffer; 
	sp<MemoryHeapBase>  	mPreviewHeap;  // format: 565
    	sp<MemoryBase>      		mPreviewBuffer;
	sp<MemoryHeapBase>  	mRecordingHeap;
    	sp<MemoryBase>      		mRecordingBuffer;
	sp<MemoryHeapBase>		mReleaseHeap;
		
    	bool                			mPreviewRunning;
    
    	notify_callback    			mNotifyCb;
    	data_callback      			mDataCb;
    	data_callback_timestamp 	mDataCbTimestamp;
    	void               			*mCallbackCookie;

    	sp<PreviewThread>   		mPreviewThread;

   	int32_t             			mMsgEnabled;
		
    	bool						previewStopped;
	bool						recordStopped;
    	V4L2Camera				camera;
	int                                	fd_video_preview;
	int                                	fd_video_picture;
	int                                	fd_video_record;
	bool						driver_used;
};
}; // namespace android

#endif
