/*
** Copyright (C) 2009.11 Embest For AT91SAM9M10-Android2.0 camera By LiuXin- http://www.embedinfo.com/
** Copyright (C) 2009 0xlab.org - http://0xlab.org/
** Copyright 2008, The Android Open Source Project
**
** Licensed under the Apache License, Version 2.0 (the "License");
** you may not use this file except in compliance with the License.
** You may obtain a copy of the License at
**
**     http://www.apache.org/licenses/LICENSE-2.0
** 
** Unless required by applicable law or agreed to in writing, software
** distributed under the License is distributed on an "AS IS" BASIS,
** WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
** See the License for the specific language governing permissions and
** limitations under the License
*/

#define LOG_TAG "V4L2Camera"
#include <utils/Log.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <sys/select.h>
#include <linux/videodev2.h>
//#include <jpeglib.h>
#include <assert.h>
#include <getopt.h>             /* getopt_long() */
#include <fcntl.h>              /* low-level i/o */
#include <unistd.h>
#include <errno.h>
#include<malloc.h>
#include<sys/stat.h>
#include<sys/types.h>
#include<sys/time.h>
#include<sys/mman.h>
#include <sys/ioctl.h>
#include <asm/types.h>

extern "C" {
    #include "jpeglib.h"
}

#include "V4L2Camera.h"

namespace android {

V4L2Camera::V4L2Camera ()
{
	fd_open = 0;
	fd_preview = 0;
    	fd_capture = 0;
    	videoIn_capture = (struct vdIn *) calloc (1, sizeof (struct vdIn));
	videoIn_preview = (struct vdIn *) calloc (1, sizeof (struct vdIn));
}

V4L2Camera::~V4L2Camera()
{

    	free(videoIn_capture);
	free(videoIn_preview);
}

int V4L2Camera::Open(int width, int height, int pixelformat,unsigned int m_preview)
{
	int ret;
	const char *device;

	if( m_preview == 1){
		device = "/dev/video1";//preview
		videoIn = videoIn_preview;
		ccRGBtoYUV = CCRGB16toYUV420::New();
		ccRGBtoYUV->Init(width, height, width, width, height, ((width + 15) >> 4) << 4, 0);
	}
    	else{
		device = "/dev/video0";//capture
		videoIn = videoIn_capture;
    	}

    	if ((fd_open = open(device, O_RDWR)) == -1) {
        	LOGE("V4L2_ERROR opening V4L interface: %s %s", device,strerror(errno));
   		goto V4L2_ERROR;
    	}
    
    	LOGE("open %s ok! fd=%d\n",device,fd_open);
	
    	ret = ioctl (fd_open, VIDIOC_QUERYCAP, &videoIn->cap);
    	if (ret < 0) {
       	LOGE("Error opening device: unable to query device.");
        	goto V4L2_ERROR;
    	}

    	if (!(videoIn->cap.capabilities & V4L2_CAP_VIDEO_CAPTURE)) {
        	LOGE("Error opening device: video capture not supported.");
        	goto V4L2_ERROR;
    	}

    	if (m_preview == 1 && !(videoIn->cap.capabilities & V4L2_CAP_STREAMING)) {
        	LOGE("Capture device does not support streaming i/o");
        	goto V4L2_ERROR;
    	}

	videoIn->width = width;
	videoIn->height = height;

	memset ( &(videoIn->format), 0, sizeof(videoIn->format) );
	videoIn->format.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
	videoIn->format.fmt.pix.pixelformat = pixelformat;
	videoIn->format.fmt.pix.field = V4L2_FIELD_INTERLACED;
	videoIn->format.fmt.pix.width = width;
	videoIn->format.fmt.pix.height = height;
	
	ret = ioctl(fd_open, VIDIOC_S_FMT, &videoIn->format);
	if (ret < 0) {
	       LOGE("Open: VIDIOC_S_FMT Failed: %s", strerror(errno));
	       goto V4L2_ERROR;
	}

	memset ( &(videoIn->format), 0, sizeof(videoIn->format));
	videoIn->format.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
	videoIn->format.fmt.pix.pixelformat = pixelformat;
	
	ret = ioctl(fd_open, VIDIOC_G_FMT, &videoIn->format);
	if (ret < 0) {
	       LOGE("Open: VIDIOC_G_FMT Failed: %s", strerror(errno));
	       goto V4L2_ERROR;
	}
	
	videoIn->framesizeIn = videoIn->format.fmt.pix.sizeimage;
	videoIn->formatIn = videoIn->format.fmt.pix.pixelformat;

	if( m_preview == 1){
		//preview
		fd_preview = fd_open;
		videoIn->isStreaming = false;
		LOGD("V4L2: preview patch open successful");
    	}
    	else{
		//capture
		fd_capture = fd_open;
		videoIn->isCapture = false;
		LOGD("V4L2: capture patch open successful");
    	}
	
    return fd_open;

V4L2_ERROR:
   
    return  UNKNOWN_ERROR;
}

void V4L2Camera::Close (int fd)
{    
	if (fd == fd_capture){
		if(close(fd_capture) ==0)
			LOGD("Close:Capture patch close OK. fd:%d",fd_capture);
		else
			LOGE("Close:Capture patch close error. fd:%d",fd_capture);
		fd_capture = -1;
	}
	else if(fd == fd_preview){
		if(close(fd_preview) ==0)
		LOGD("Close:Preview patch close OK. fd:%d",fd_preview);
		else
			LOGE("Close:Preview patch close error. fd:%d",fd_preview);
		fd_preview = -1;
	}
	else{
		LOGE("Close:No such device. fd:%d",fd);
	}
}



int V4L2Camera::Init(int fd)
{
    	int ret;

	if (fd == fd_capture){
		videoIn = videoIn_capture;
		videoIn->mem[0] = malloc (videoIn->framesizeIn);
		if(!videoIn->mem[0]){
			LOGE("Init:malloc buffer failed for capture path");
			goto V4L2_ERROR;
		}
	}
	else if (fd == fd_preview){
		videoIn = videoIn_preview;
		/* Check if camera can handle NB_BUFFER buffers */
	    	videoIn->rb.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
	    	videoIn->rb.memory = V4L2_MEMORY_MMAP;
	    	videoIn->rb.count = NB_BUFFER;

	    	ret = ioctl(fd, VIDIOC_REQBUFS, &videoIn->rb);
	    	if (ret < 0) {
	       	LOGE("Init: VIDIOC_REQBUFS failed: %s", strerror(errno));
	       	goto V4L2_ERROR;
	    	}
	    	LOGD("REQBUF Count=%d \n",videoIn->rb.count);
	    	for (int i = 0; i < videoIn->rb.count/*NB_BUFFER*/; i++) {
			memset (&videoIn->buf, 0, sizeof (struct v4l2_buffer));
			videoIn->buf.index = i;
	        	videoIn->buf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
	        	videoIn->buf.memory = V4L2_MEMORY_MMAP;

	        	ret = ioctl (fd, VIDIOC_QUERYBUF, &videoIn->buf);
	        	if (ret < 0) {
	            		LOGE("Init: Unable to query buffer (%s)", strerror(errno));
	            		goto V4L2_ERROR;
	        	}
		
	        	 videoIn->mem[i] = mmap (0,
	               videoIn->buf.length,
	               PROT_READ | PROT_WRITE,
	               MAP_SHARED,
	               fd,
	               videoIn->buf.m.offset);

		if (videoIn->mem[i] == MAP_FAILED) {
			LOGE("Init: Unable to map buffer (%s)", strerror(errno));
	       	goto V4L2_ERROR;
	        }
			
		ret = ioctl(fd, VIDIOC_QBUF, &videoIn->buf);
	       if (ret < 0) {
	            LOGE("Init: VIDIOC_QBUF Failed");
	            goto V4L2_ERROR;
	        }	
	}	

    }

    return 0;
V4L2_ERROR:
 
    return  UNKNOWN_ERROR;
}

void V4L2Camera::Uninit(int fd)
{
	int ret;

	if (fd == fd_capture){
		videoIn = videoIn_capture;
		free(videoIn->mem[0]);
	}
	else if (fd == fd_preview){
		videoIn = videoIn_preview;
		videoIn->buf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
		videoIn->buf.memory = V4L2_MEMORY_MMAP;
		
		/* Unmap buffers */
		for (int i = 0; i < videoIn->rb.count/*NB_BUFFER*/; i++)
			if (munmap(videoIn->mem[i], videoIn->buf.length) < 0)
				LOGE("Uninit: Unmap failed");
	}
	else{
		LOGE("Uninit:No such device. fd:%d",fd);
	}


}

int V4L2Camera::StartStreaming (int fd)
{
    	enum v4l2_buf_type type;
    	int ret;
    	int fd_current;

	if (fd == fd_capture)
		fd_current = 0;
	else if (fd == fd_preview)
		fd_current = 1;
	else
		fd_current = 2;
	
	switch (fd_current){
		case 0:
			videoIn = videoIn_capture;
			videoIn->isCapture = true;
			LOGD("StartStreaming: Start capture successful");
		break;
		case 1:
			videoIn = videoIn_preview;
			if (!videoIn->isStreaming) {
        		type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
			ret = ioctl (fd, VIDIOC_STREAMON, &type);
        			if (ret < 0) {
            				LOGE("StartStreaming: Unable to start preview: %s", strerror(errno));
            				return ret;
        			}
				else{
					LOGD("StartStreaming: Start preview successful");
				}
			}
			videoIn->isStreaming = true;
		break;
		default:
			LOGE("StartStreaming:no such device");
			return 1;
    }
	return 0;
}

int V4L2Camera::StopStreaming (int fd)
{   
    	enum v4l2_buf_type type;
    	int ret;
    	int fd_current;

	if (fd == fd_capture)
		fd_current = 0;
	else if (fd == fd_preview)
		fd_current = 1;
	else
		fd_current = 2;
	
   	 switch (fd_current){
		case 0:
			videoIn = videoIn_capture;
			videoIn->isCapture = false;
			LOGD("StopStreaming: Stop capture successful");
		break;
		case 1:
			videoIn = videoIn_preview;
			if (videoIn->isStreaming) {
        		type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
			ret = ioctl (fd, VIDIOC_STREAMOFF, &type);
        			if (ret < 0) {
            				LOGE("StopStreaming: Unable to stop preview: %s", strerror(errno));
            				return ret;
        			}
				else{
					LOGD("StopStreaming: Stop preview successful");
				}
			}
			videoIn->isStreaming = false;
			break;
		default:
			LOGE("StopStreaming:no such file device");
			return 1;
    }
	return 0;
}

int V4L2Camera::GrabRawFrame(void *previewBuffer,int fd)
{	
    	int ret=0;

	if (fd == fd_capture)
		videoIn = videoIn_capture;
	else if (fd == fd_preview)
		videoIn = videoIn_preview;
	
	videoIn->buf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
	videoIn->buf.memory = V4L2_MEMORY_MMAP;
	videoIn->buf.index = 0;
	
	/* DQ */
	do{		
		ret = ioctl(fd, VIDIOC_DQBUF, &videoIn->buf);
		if (ret != 0) {
			LOGD("GrabRawFrame: VIDIOC_DQBUF Failed:%s,fd=%d,index=%d\n",strerror(errno),fd,videoIn->buf.index);
		}
	}while(ret != 0);

	memcpy(previewBuffer, videoIn->mem[videoIn->buf.index], (size_t) (videoIn->width * videoIn->height * 2));

	do{
		ret = ioctl(fd, VIDIOC_QBUF, &videoIn->buf);
		if (ret != 0) {
			LOGD("GrabRawFrame: VIDIOC_QBUF Failed:%s,fd=%d,index=%d\n",strerror(errno),fd,videoIn->buf.index);
		}
	}while(ret != 0);
		
	return 0;
    
}

int 
V4L2Camera::savePicture(unsigned char *inputBuffer, const char * filename)
{
    FILE *output;
    int fileSize;
    int ret;
    output = fopen(filename, "wb");

    if (output == NULL) {
        LOGE("GrabJpegFrame: Ouput file == NULL");
        return 0;
    }

    fileSize = saveYUYVtoJPEG(inputBuffer, videoIn->width, videoIn->height, output, 100);
	
    fclose(output);

    LOGD("savePicture: saveYUYVtoJPEG OK\n");

    return fileSize;
}

int V4L2Camera::GrabJpegFrame (int fd)
{ 
   
    	FILE *output;
    	int fileSize;
    	int ret;
    	int index;
    	int i=0;
    	unsigned int QBUF_OK = 1;

	if (fd == fd_capture)
		videoIn = videoIn_capture;
	else if (fd == fd_preview)
		videoIn = videoIn_preview;

	videoIn->buf.type = V4L2_BUF_TYPE_VIDEO_CAPTURE;
	videoIn->buf.memory = V4L2_MEMORY_MMAP;
	videoIn->buf.index = 0;
		
	do{
		ret = read (fd, videoIn->mem[0], videoIn->framesizeIn);
	    	if (ret != videoIn->framesizeIn) {
	        	LOGE("GrabJpegFrame: read capture framebuffer Failed,fd=%d\n",fd);
	    	}
		LOGE("ret is %d",ret);
       }while(ret < 0);
	
	fileSize = savePicture((unsigned char *)videoIn->mem[0], "/tmp/tmp.jpeg");

	return fileSize;
   
}

int V4L2Camera::readjpeg(void *previewBuffer,int fileSize)
{
	   FILE *input;
	   input = fopen("/tmp/tmp.jpeg", "rb");
	   if (input == NULL)
	        LOGE("readjpeg: Input file == NULL");
	   else if(previewBuffer == NULL)
	   	LOGE("readjpeg: previewBuffer == NULL");
	   else {
	        fread((uint8_t *)previewBuffer, 1, fileSize, input);
	        fclose(input);
		 LOGD("read jpeg OK");
	        return 0;
	    }
	   return fileSize;
}

int V4L2Camera::saveYUYVtoJPEG (unsigned char *inputBuffer, int width, int height, FILE *file, int quality)
{	
   	struct jpeg_compress_struct cinfo;
    	struct jpeg_error_mgr jerr;
    	JSAMPROW row_pointer[1];
    	unsigned char *line_buffer;
    	unsigned char Y1, Y2, U, V;
    	register int r,g,b;
    	unsigned int y=0;
    	int line,col;

    	int fileSize;

    	line_buffer = (unsigned char *) calloc (width *3, 1);
	
    	cinfo.err = jpeg_std_error (&jerr);
    	jpeg_create_compress (&cinfo);
    	jpeg_stdio_dest (&cinfo, file);

    	cinfo.image_width = width;
    	cinfo.image_height = height;
    	cinfo.input_components = 3;
    	cinfo.in_color_space = JCS_RGB;

    	jpeg_set_defaults (&cinfo);
    	jpeg_set_quality (&cinfo, quality, TRUE);

    	jpeg_start_compress (&cinfo, TRUE);

   	for (line = 0; line < height; line++) {
		unsigned char *ptr = line_buffer;
	       for (col = 0; col < width; col+=2) {
		   	U = inputBuffer[y + 0];
		       Y1 = inputBuffer[y + 1];
		       V = inputBuffer[y + 2];
		       Y2 = inputBuffer[y + 3];

			r = (1192 * (Y1 - 16) + 1634 * (V - 128) ) >> 10;
			g = (1192 * (Y1 - 16) - 833 * (V - 128) - 400 * (U -128) ) >> 10;
			b = (1192 * (Y1 - 16) + 2066 * (U - 128) ) >> 10;
	
		       *(ptr++) = (r > 255) ? 255 : ((r < 0) ? 0 : r);
	              *(ptr++) = (g > 255) ? 255 : ((g < 0) ? 0 : g);
	              *(ptr++) = (b > 255) ? 255 : ((b < 0) ? 0 : b);

			r = (1192 * (Y2 - 16) + 1634 * (V - 128) ) >> 10;
			g = (1192 * (Y2 - 16) - 833 * (V - 128) - 400 * (U -128) ) >> 10;
			b = (1192 * (Y2 - 16) + 2066 * (U - 128) ) >> 10;

		       *(ptr++) = (r > 255) ? 255 : ((r < 0) ? 0 : r);
	              *(ptr++) = (g > 255) ? 255 : ((g < 0) ? 0 : g);
	              *(ptr++) = (b > 255) ? 255 : ((b < 0) ? 0 : b);

		        y = y+4;
	        }//line end	 	 

	        row_pointer[0] = line_buffer;
	        jpeg_write_scanlines (&cinfo, row_pointer, 1);
	}

    	jpeg_finish_compress (&cinfo);
    	fileSize = ftell(file);
    	jpeg_destroy_compress (&cinfo);

    	free (line_buffer);

    return fileSize;

}

static inline void yuv_to_rgb16(unsigned char y,
                                unsigned char u,
                                unsigned char v,
                                unsigned char *rgb)
{
    	register int r,g,b;
    	int rgb16;

    	r = (1192 * (y - 16) + 1634 * (v - 128) ) >> 10;
    	g = (1192 * (y - 16) - 833 * (v - 128) - 400 * (u -128) ) >> 10;
    	b = (1192 * (y - 16) + 2066 * (u - 128) ) >> 10;

    	r = r > 255 ? 255 : r < 0 ? 0 : r;
    	g = g > 255 ? 255 : g < 0 ? 0 : g;
    	b = b > 255 ? 255 : b < 0 ? 0 : b;

    	rgb16 = (int)(((r >> 3)<<11) | ((g >> 2) << 5)| ((b >> 3) << 0));

    	*rgb = (unsigned char)(rgb16 & 0xFF);
    	rgb++;
    	*rgb = (unsigned char)((rgb16 & 0xFF00) >> 8);

}

void V4L2Camera::convert(unsigned char *buf, unsigned short *rgb_in, int width, int height)
{ 
   	int x,y,z=0;
    	int blocks;
    	static int count_in = 0;

  	count_in ++ ;	

   	unsigned char *rgb = (unsigned char *) rgb_in;	

    	blocks = (width * height) * 2;

    	for (y = 0; y < blocks; y+=4) {
       	unsigned char Y1, Y2, U, V;

        	U = buf[y + 0];
        	Y1 = buf[y + 1];
        	V = buf[y + 2];
        	Y2 = buf[y + 3];

        	yuv_to_rgb16(Y1, U, V, &rgb[y]);
        	yuv_to_rgb16(Y2, U, V, &rgb[y + 2]);
	}
}

void V4L2Camera::rgb16TOyuv420(unsigned char *rgb16, unsigned char *yuv420)
{ 
	ccRGBtoYUV->Convert(rgb16, yuv420);
}
}; // namespace android
/*
 * 2010.04.17 Final V1 ,Embest Liu Xin
*/
