package com.android.calibrate;

import android.app.Activity;
import com.android.calibrate.R;
import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.os.Bundle;
import android.util.Log;
import android.view.MotionEvent;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;
import android.view.View.OnTouchListener;
import android.widget.Toast;
import android.os.SystemProperties;

public class AndroidCalibrate extends Activity {
	final String TAG = "TSCalibration";
	final int UI_SCREEN_WIDTH = 480;
	final int UI_SCREEN_HEIGHT = 272;
	int direction;
	CrossView myview;

	private Calibrate cal;

	int xList[] = { 50, UI_SCREEN_WIDTH - 50, UI_SCREEN_WIDTH - 50, 50, UI_SCREEN_WIDTH / 2 };
	int yList[] = { 50, 50, UI_SCREEN_HEIGHT - 50, UI_SCREEN_HEIGHT - 50, UI_SCREEN_HEIGHT / 2 };

	private void setFullScreen() {
		requestWindowFeature(Window.FEATURE_NO_TITLE);
		getWindow().setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN, WindowManager.LayoutParams.FLAG_FULLSCREEN);
	}

	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setFullScreen();
		myview = new CrossView(this);
		setContentView(myview);

		SystemProperties.set("dev.ts.calibrate", "start");
		String prop = SystemProperties.get("dev.ts.calibrate", "null");
		if (false == prop.equalsIgnoreCase("start"))
			Log.e("prop set failed", prop);

		cal = new Calibrate();
		direction = 0;

		myview.setOnTouchListener(new OnTouchListener() {

			public boolean onTouch(View v, MotionEvent event) {
				int X = (int) event.getRawX();
				int Y = (int) event.getRawY();
				Log.i("OnTouch", X + "," + Y);
				v.invalidate();
				if (direction <= 4) {
					Log.i("TS_Calibration time onTouchListener", " " + direction);
					cal.get_sample(direction, X, Y, xList[direction], yList[direction]);
				}
				if (direction == 4) {					
					cal.calibrate_main();

					SystemProperties.set("dev.ts.calibrate", "done");
					String prop = SystemProperties.get("dev.ts.calibrate", "null");
					if (false == prop.equalsIgnoreCase("done"))
						Log.e("prop set failed", prop);

					Toast.makeText(getBaseContext(), "Calibrate Done!", Toast.LENGTH_SHORT).show();

					AndroidCalibrate.this.finish();
				}
				direction++;
				return false;
			}
		});
	}

	public class CrossView extends View {
		public CrossView(Context context) {
			super(context);
		}

		public void onDraw(Canvas canvas) {

			Paint paint = new Paint();
			paint.setColor(Color.GREEN);
			if (direction == 0) {
				canvas.drawLine(40, 50, 60, 50, paint);
				canvas.drawLine(50, 40, 50, 60, paint);
				paint.setColor(Color.WHITE);
			} else if (direction == 1) {
				canvas.drawLine(UI_SCREEN_WIDTH - 60, 50, UI_SCREEN_WIDTH - 40, 50, paint);
				canvas.drawLine(UI_SCREEN_WIDTH - 50, 40, UI_SCREEN_WIDTH - 50, 60, paint);
				paint.setColor(Color.WHITE);

			} else if (direction == 2) {
				canvas.drawLine(UI_SCREEN_WIDTH - 60, UI_SCREEN_HEIGHT - 50, UI_SCREEN_WIDTH - 40, UI_SCREEN_HEIGHT - 50, paint);
				canvas.drawLine(UI_SCREEN_WIDTH - 50, UI_SCREEN_HEIGHT - 60, UI_SCREEN_WIDTH - 50, UI_SCREEN_HEIGHT - 40, paint);
				paint.setColor(Color.WHITE);
			} else if (direction == 3) {
				canvas.drawLine(40, UI_SCREEN_HEIGHT - 50, 60, UI_SCREEN_HEIGHT - 50, paint);
				canvas.drawLine(50, UI_SCREEN_HEIGHT - 60, 50, UI_SCREEN_HEIGHT - 40, paint);
				paint.setColor(Color.WHITE);
			} else if (direction == 4) {
				canvas.drawLine(UI_SCREEN_WIDTH / 2 - 10, UI_SCREEN_HEIGHT / 2, UI_SCREEN_WIDTH / 2 + 10, UI_SCREEN_HEIGHT / 2, paint);
				canvas.drawLine(UI_SCREEN_WIDTH / 2, UI_SCREEN_HEIGHT / 2 - 10, UI_SCREEN_WIDTH / 2, UI_SCREEN_HEIGHT / 2 + 10, paint);
				paint.setColor(Color.WHITE);
			}
			canvas.drawText(getResources().getString(R.string.app_name), UI_SCREEN_WIDTH / 2 - 50, UI_SCREEN_HEIGHT / 2, paint);
			super.onDraw(canvas);
		}
	}

}
